@@ -15,8 +15,6 @@
 
 import os
 import re
-import shutil
-import tempfile
 import time
 
 import jinja2
@@ -80,13 +78,17 @@ class Deployment(object):
         # TODO(akscram): Check that the deployment have got a status that
         #                is equal to "*->finished" or "deploy->inconsistent".
         deployment = objects.Deployment.get(deployment)
-        deployer = deploy_engine.Engine.get_engine(
-            deployment["config"]["type"], deployment)
-
         tempest.Tempest(deployment["uuid"]).uninstall()
-        with deployer:
-            deployer.make_cleanup()
-            deployment.delete()
+        try:
+            deployer = deploy_engine.Engine.get_engine(
+                deployment["config"]["type"], deployment)
+            with deployer:
+                deployer.make_cleanup()
+        except exceptions.PluginNotFound:
+            LOG.info(_("Deployment %s will be deleted despite"
+                       " exception") % deployment["uuid"])
+
+        deployment.delete()
 
     @classmethod
     def recreate(cls, deployment):
@@ -368,6 +370,15 @@ class Task(object):
 
 class Verification(object):
 
+    @staticmethod
+    def _check_tempest_tree_existence(verifier):
+        if not os.path.exists(verifier.path()):
+            msg = _("Tempest tree for "
+                    "deployment '%s' not found! ") % verifier.deployment
+            LOG.error(
+                msg + _("Use `rally verify install` for Tempest installation"))
+            raise exceptions.NotFoundException(message=msg)
+
     @classmethod
     def verify(cls, deployment, set_name="", regex=None, tests_file=None,
                tempest_config=None, expected_failures=None, system_wide=False,
@@ -401,17 +412,7 @@ class Verification(object):
                                    tempest_config=tempest_config,
                                    system_wide=system_wide)
 
-        if not verifier.is_installed():
-            LOG.warning("Installation of Tempest will be deprecated and "
-                        "removed in the future when executing the `rally "
-                        "verify start` command. To install Tempest please "
-                        "start to use the `rally verify install` command "
-                        "before `rally verify start`.")
-            LOG.info(_("Tempest is not installed "
-                       "for the specified deployment."))
-            LOG.info(_("Installing Tempest "
-                       "for deployment: %s") % deployment_uuid)
-            verifier.install()
+        cls._check_tempest_tree_existence(verifier)
 
         LOG.info("Starting verification of deployment: %s" % deployment_uuid)
         verification.set_running()
@@ -443,16 +444,19 @@ class Verification(object):
         return deployment, verification
 
     @classmethod
-    def install_tempest(cls, deployment, source=None, system_wide=False):
+    def install_tempest(cls, deployment, source=None, version=None,
+                        system_wide=False):
         """Install Tempest.
 
         :param deployment: UUID or name of a deployment
         :param source: Path/URL to repo to clone Tempest from
+        :param version: Commit ID or tag to checkout before Tempest
+                        installation
         :param system_wide: Whether or not to create a Tempest virtual env
         """
         deployment_uuid = objects.Deployment.get(deployment)["uuid"]
         verifier = tempest.Tempest(deployment_uuid, source=source,
-                                   system_wide=system_wide)
+                                   version=version, system_wide=system_wide)
         verifier.install()
 
     @classmethod
@@ -466,31 +470,43 @@ class Verification(object):
         verifier.uninstall()
 
     @classmethod
-    def reinstall_tempest(cls, deployment, tempest_config=None,
-                          source=None, system_wide=False):
+    def reinstall_tempest(cls, deployment, source=None, version=None,
+                          system_wide=False):
         """Uninstall Tempest and install again.
 
         :param deployment: UUID or name of a deployment
-        :param tempest_config: User specified Tempest config file location
         :param source: Path/URL to repo to clone Tempest from
+        :param version: Commit ID or tag to checkout before Tempest
+                        installation
         :param system_wide: Whether or not to create a Tempest virtual env
         """
         deployment_uuid = objects.Deployment.get(deployment)["uuid"]
-        verifier = tempest.Tempest(deployment_uuid)
-        if not tempest_config:
-            config_path = verifier.config_file
-            filename = os.path.basename(config_path)
-            temp_location = tempfile.gettempdir()
-            tmp_conf_path = os.path.join(temp_location, filename)
-            shutil.copy2(config_path, tmp_conf_path)
-        source = source or verifier.tempest_source
-        verifier.uninstall()
         verifier = tempest.Tempest(deployment_uuid, source=source,
-                                   tempest_config=tempest_config,
-                                   system_wide=system_wide)
+                                   version=version, system_wide=system_wide)
+        verifier.uninstall()
         verifier.install()
-        if not tempest_config:
-            shutil.move(tmp_conf_path, verifier.config_file)
+
+    @classmethod
+    def install_tempest_plugin(cls, deployment, source=None, version=None,
+                               system_wide=False):
+        """Install Tempest plugin.
+
+        :param deployment: UUID or name of a deployment
+        :param source: Path/URL to repo to clone Tempest plugin from
+        :param version: Branch, commit ID or tag to checkout before Tempest
+                        plugin installation
+        :param system_wide: Install plugin in Tempest virtual env or
+                            in the local env
+        """
+        deployment_uuid = objects.Deployment.get(deployment)["uuid"]
+        verifier = tempest.Tempest(deployment_uuid,
+                                   plugin_source=source,
+                                   plugin_version=version,
+                                   system_wide=system_wide)
+
+        cls._check_tempest_tree_existence(verifier)
+
+        verifier.install_plugin()
 
     @classmethod
     def discover_tests(cls, deployment, pattern=""):
@@ -501,16 +517,10 @@ class Verification(object):
         """
         deployment_uuid = objects.Deployment.get(deployment)["uuid"]
         verifier = tempest.Tempest(deployment_uuid)
-        return verifier.discover_tests(pattern)
 
-    @staticmethod
-    def _check_tempest_tree_existence(verifier):
-        if not os.path.exists(verifier.path()):
-            msg = _("Tempest tree for "
-                    "deployment '%s' not found! ") % verifier.deployment
-            LOG.error(
-                msg + _("Use `rally verify install` for Tempest installation"))
-            raise exceptions.NotFoundException(message=msg)
+        cls._check_tempest_tree_existence(verifier)
+
+        return verifier.discover_tests(pattern)
 
     @classmethod
     def configure_tempest(cls, deployment, tempest_config=None,
