@@ -132,15 +132,14 @@ class TaskCommands(object):
 
     def _load_and_validate_task(self, task, task_args, task_args_file,
                                 deployment, task_instance=None):
-        if not os.path.isfile(task):
-            err_cls = IOError
-            msg = "No such file '%s'" % task
+        try:
+            input_task = self._load_task(task, task_args, task_args_file)
+        except Exception as err:
             if task_instance:
-                task_instance.set_failed(err_cls.__name__,
-                                         msg,
+                task_instance.set_failed(err.__class__.__name__,
+                                         str(err),
                                          json.dumps(traceback.format_stack()))
-            raise err_cls(msg)
-        input_task = self._load_task(task, task_args, task_args_file)
+            raise
         api.Task.validate(deployment, input_task, task_instance)
         print(_("Task config is valid :)"))
         return input_task
@@ -350,7 +349,6 @@ class TaskCommands(object):
             print(json.dumps(key["kw"], indent=2))
             print()
 
-            durations = plot.charts.MainStatsTable(result["info"])
             iterations = []
             iterations_headers = ["iteration", "full duration"]
             iterations_actions = []
@@ -363,7 +361,6 @@ class TaskCommands(object):
                     iterations_actions.append((atomic_name, action))
 
             for idx, itr in enumerate(result["iterations"], 1):
-                durations.add_iteration(itr)
 
                 if iterations_data:
                     row = {"iteration": idx,
@@ -400,11 +397,11 @@ class TaskCommands(object):
             self._print_task_errors(task_id, task_errors)
 
             cols = plot.charts.MainStatsTable.columns
-            float_cols = cols[1:7]
+            float_cols = result["info"]["stat"]["cols"][1:7]
             formatters = dict(zip(float_cols,
                                   [cliutils.pretty_float_formatter(col, 3)
                                    for col in float_cols]))
-            rows = [dict(zip(cols, r)) for r in durations.render()["rows"]]
+            rows = [dict(zip(cols, r)) for r in result["info"]["stat"]["rows"]]
             cliutils.print_list(rows,
                                 fields=cols,
                                 formatters=formatters,
@@ -546,6 +543,67 @@ class TaskCommands(object):
                 print(_("There are no tasks. To run a new task, use:\n"
                         "\trally task start"))
 
+    @cliutils.args("--out", metavar="<path>",
+                   type=str, dest="out", required=False,
+                   help="Path to output file.")
+    @cliutils.args("--open", dest="open_it", action="store_true",
+                   help="Open the output in a browser.")
+    @cliutils.args("--tasks", dest="tasks", nargs="+",
+                   help="UUIDs of tasks, or JSON files with task results")
+    @cliutils.suppress_warnings
+    def trends(self, *args, **kwargs):
+        """Generate workloads trends HTML report."""
+        tasks = kwargs.get("tasks", []) or list(args)
+
+        if not tasks:
+            print(_("ERROR: At least one task must be specified"),
+                  file=sys.stderr)
+            return 1
+
+        results = []
+        for task_id in tasks:
+            if os.path.exists(os.path.expanduser(task_id)):
+                with open(os.path.expanduser(task_id), "r") as inp_js:
+                    task_results = json.load(inp_js)
+                    for result in task_results:
+                        try:
+                            jsonschema.validate(
+                                result,
+                                api.Task.TASK_RESULT_SCHEMA)
+                        except jsonschema.ValidationError as e:
+                            print(_("ERROR: Invalid task result format in %s")
+                                  % task_id, file=sys.stderr)
+                            print(six.text_type(e), file=sys.stderr)
+                            return 1
+
+            elif uuidutils.is_uuid_like(task_id):
+                task_results = map(
+                    lambda x: {"key": x["key"],
+                               "sla": x["data"]["sla"],
+                               "result": x["data"]["raw"],
+                               "load_duration": x["data"]["load_duration"],
+                               "full_duration": x["data"]["full_duration"]},
+                    api.Task.get(task_id).get_results())
+            else:
+                print(_("ERROR: Invalid UUID or file name passed: %s")
+                      % task_id, file=sys.stderr)
+                return 1
+
+            results.extend(task_results)
+
+        result = plot.trends(results)
+
+        out = kwargs.get("out")
+        if out:
+            output_file = os.path.expanduser(out)
+
+            with open(output_file, "w+") as f:
+                f.write(result)
+            if kwargs.get("open_it"):
+                webbrowser.open_new_tab("file://" + os.path.realpath(out))
+        else:
+            print(result)
+
     @cliutils.args("--tasks", dest="tasks", nargs="+",
                    help="UUIDs of tasks, or JSON files with task results")
     @cliutils.args("--out", metavar="<path>",
@@ -739,12 +797,12 @@ class TaskCommands(object):
 
         parsed_obj = urlparse.urlparse(connection_string)
         try:
-            client = exporter.TaskExporter.get(parsed_obj.scheme)(
+            client = exporter.Exporter.get(parsed_obj.scheme)(
                 connection_string)
         except exceptions.InvalidConnectionString as e:
             if logging.is_debug():
                 LOG.exception(e)
-            print (e)
+            print(e)
             return 1
         except exceptions.PluginNotFound as e:
             if logging.is_debug():
@@ -752,7 +810,7 @@ class TaskCommands(object):
             msg = ("\nPlease check your connection string. The format of "
                    "`connection` should be plugin-name://"
                    "<user>:<pwd>@<full_address>:<port>/<path>.<type>")
-            print (str(e) + msg)
+            print(str(e) + msg)
             return 1
 
         try:
@@ -760,7 +818,7 @@ class TaskCommands(object):
         except (IOError, exceptions.RallyException) as e:
             if logging.is_debug():
                 LOG.exception(e)
-            print (e)
+            print(e)
             return 1
         print(_("Task %(uuid)s results was successfully exported to %("
                 "connection)s using %(name)s plugin.") % {
