@@ -71,17 +71,17 @@ class VerifyCommands(object):
                         "requirements have to be already installed in "
                         "the local env!",
                    required=False, action="store_true")
-    @cliutils.deprecated_args("--system-wide-install", dest="system_wide",
-                              required=False, action="store_true",
-                              release="0.1.2", alternative="--system-wide")
     @cliutils.args("--concurrency", metavar="N", dest="concur", type=int,
                    required=False,
                    help="How many processes to use to run Tempest tests. "
                         "The default value (0) auto-detects your CPU count")
+    @cliutils.args("--failing", dest="failing", required=False,
+                   help="Re-run the tests that failed in the last execution",
+                   action="store_true")
     @envutils.with_default_deployment(cli_arg_name="deployment")
     def start(self, deployment=None, set_name="", regex=None,
               tests_file=None, tempest_config=None, xfails_file=None,
-              do_use=True, system_wide=False, concur=0):
+              do_use=True, system_wide=False, concur=0, failing=False):
         """Start verification (run Tempest tests).
 
         :param deployment: UUID or name of a deployment
@@ -98,7 +98,9 @@ class VerifyCommands(object):
                             env when running the tests
         :param concur: How many processes to use to run Tempest tests.
                        The default value (0) auto-detects CPU count
+        :param failing: Re-run tests that failed during the last execution
         """
+
         msg = _("Arguments '%s' and '%s' are not compatible. "
                 "You can use only one of the mentioned arguments.")
         if regex and set_name:
@@ -111,7 +113,7 @@ class VerifyCommands(object):
             print(msg % ("tests_file", "regex"))
             return 1
 
-        if not (regex or set_name or tests_file):
+        if not (regex or set_name or tests_file or failing):
             set_name = "full"
 
         if set_name and set_name not in AVAILABLE_SETS:
@@ -124,6 +126,14 @@ class VerifyCommands(object):
             print(_("File '%s' not found.") % tests_file)
             return 1
 
+        if failing and set_name:
+            print(msg % ("failing", "set"))
+            return 1
+
+        if failing and tests_file:
+            print(msg % ("failing", "tests_file"))
+            return 1
+
         expected_failures = None
         if xfails_file:
             if os.path.exists(xfails_file):
@@ -136,9 +146,12 @@ class VerifyCommands(object):
         verification = api.Verification.verify(
             deployment, set_name=set_name, regex=regex, tests_file=tests_file,
             tempest_config=tempest_config, expected_failures=expected_failures,
-            system_wide=system_wide, concur=concur)
+            system_wide=system_wide, concur=concur, failing=failing)
+
         if do_use:
             self.use(verification["uuid"])
+        else:
+            print(_("Verification UUID: %s") % verification["uuid"])
 
     @cliutils.args("--deployment", dest="deployment", type=str,
                    metavar="<uuid>", required=False,
@@ -170,6 +183,8 @@ class VerifyCommands(object):
                                                                    log_file)
         if do_use:
             self.use(verification["uuid"])
+        else:
+            print(_("Verification UUID: %s") % verification["uuid"])
 
     def list(self):
         """List verification runs."""
@@ -183,6 +198,7 @@ class VerifyCommands(object):
 
         if verifications:
             cliutils.print_list(verifications, fields,
+                                normalize_field_names=True,
                                 sortby_index=fields.index("Created at"))
         else:
             print(_("No verification was started yet. "
@@ -257,7 +273,8 @@ class VerifyCommands(object):
         print(_("Total results of verification:\n"))
         total_fields = ["UUID", "Deployment UUID", "Set name", "Tests",
                         "Failures", "Created at", "Status"]
-        cliutils.print_list([verification], fields=total_fields)
+        cliutils.print_list([verification], fields=total_fields,
+                            normalize_field_names=True)
 
         print(_("\nTests:\n"))
         fields = ["name", "time", "status"]
@@ -355,10 +372,6 @@ class VerifyCommands(object):
 
     @cliutils.args("--uuid", type=str, dest="verification",
                    required=False, help="UUID of a verification")
-    @cliutils.deprecated_args("--verification", dest="verification",
-                              required=False, type=str,
-                              release="0.2.0",
-                              alternative="--uuid")
     def use(self, verification):
         """Set active verification.
 
@@ -394,23 +407,27 @@ class VerifyCommands(object):
                    help="UUID or name of a deployment.")
     @cliutils.args("--source", type=str, dest="source", required=False,
                    help="Path/URL to repo to clone Tempest from")
+    @cliutils.args("--version", type=str, dest="version", required=False,
+                   help="Commit ID or tag to checkout before Tempest "
+                        "installation")
     @cliutils.args("--system-wide", dest="system_wide",
                    help="Don't create a virtual env for Tempest. Note "
                         "that all Tempest requirements have to be already "
                         "installed in the local env!",
                    required=False, action="store_true")
-    @cliutils.deprecated_args("--no-tempest-venv", dest="system_wide",
-                              required=False, action="store_true",
-                              release="0.2.0", alternative="--system-wide")
     @envutils.with_default_deployment(cli_arg_name="deployment")
-    def install(self, deployment=None, source=None, system_wide=False):
+    def install(self, deployment=None, source=None, version=None,
+                system_wide=False):
         """Install Tempest.
 
         :param deployment: UUID or name of a deployment
         :param source: Path/URL to repo to clone Tempest from
+        :param version: Commit ID or tag to checkout before Tempest
+                        installation
         :param system_wide: Whether or not to create a Tempest virtual env
         """
-        api.Verification.install_tempest(deployment, source, system_wide)
+        api.Verification.install_tempest(deployment, source,
+                                         version, system_wide)
 
     @cliutils.args("--deployment", dest="deployment", type=str,
                    metavar="<uuid>", required=False,
@@ -426,31 +443,57 @@ class VerifyCommands(object):
     @cliutils.args("--deployment", dest="deployment", type=str,
                    metavar="<uuid>", required=False,
                    help="UUID or name of a deployment.")
-    @cliutils.args("--tempest-config", dest="tempest_config", type=str,
-                   required=False, metavar="<path>",
-                   help="User specified Tempest config file location")
     @cliutils.args("--source", type=str, dest="source", required=False,
                    help="Path/URL to repo to clone Tempest from")
+    @cliutils.args("--version", type=str, dest="version", required=False,
+                   help="Commit ID or tag to checkout before Tempest "
+                        "installation")
     @cliutils.args("--system-wide", dest="system_wide",
                    help="Don't create a virtual env for Tempest. Note "
                         "that all Tempest requirements have to be already "
                         "installed in the local env!",
                    required=False, action="store_true")
-    @cliutils.deprecated_args("--no-tempest-venv", dest="system_wide",
-                              required=False, action="store_true",
-                              release="0.2.0", alternative="--system-wide")
     @envutils.with_default_deployment(cli_arg_name="deployment")
-    def reinstall(self, deployment=None,
-                  tempest_config=None, source=None, system_wide=False):
+    def reinstall(self, deployment=None, source=None, version=None,
+                  system_wide=False):
         """Uninstall Tempest and install again.
 
         :param deployment: UUID or name of a deployment
-        :param tempest_config: User specified Tempest config file location
         :param source: Path/URL to repo to clone Tempest from
+        :param version: Commit ID or tag to checkout before Tempest
+                        installation
         :param system_wide: Whether or not to create a Tempest virtual env
         """
-        api.Verification.reinstall_tempest(deployment, tempest_config,
-                                           source, system_wide)
+        api.Verification.reinstall_tempest(deployment, source,
+                                           version, system_wide)
+
+    @cliutils.args("--deployment", type=str, dest="deployment",
+                   required=False, help="UUID or name of a deployment")
+    @cliutils.args("--source", type=str, dest="source", required=True,
+                   help="Path/URL to repo to clone Tempest plugin from")
+    @cliutils.args("--version", type=str, dest="version", required=False,
+                   help="Branch, commit ID or tag to checkout before Tempest "
+                        "plugin installation")
+    @cliutils.args("--system-wide", dest="system_wide",
+                   help="Install plugin in the local env, "
+                        "not in Tempest virtual env. Note that all Tempest "
+                        "plugin requirements have to be already installed in "
+                        "the local env!",
+                   required=False, action="store_true")
+    @envutils.with_default_deployment(cli_arg_name="deployment")
+    def installplugin(self, deployment=None, source=None, version=None,
+                      system_wide=False):
+        """Install Tempest plugin.
+
+        :param deployment: UUID or name of a deployment
+        :param source: Path/URL to repo to clone Tempest plugin from
+        :param version: Branch, commit ID or tag to checkout before Tempest
+                        plugin installation
+        :param system_wide: Install plugin in Tempest virtual env or
+                            in the local env
+        """
+        api.Verification.install_tempest_plugin(deployment, source,
+                                                version, system_wide)
 
     @cliutils.args("--deployment", dest="deployment", type=str, required=False,
                    metavar="<uuid>", help="UUID or name of a deployment")
