@@ -25,18 +25,19 @@ from rally.common import utils
 
 
 class _TempestTestsAPI(utils.ImmutableMixin, utils.EnumMixin):
-    BAREMTAL = "baremetal"
+    BAREMETAL = "baremetal"
+    CLUSTERING = "clustering"
     COMPUTE = "compute"
-    DNS = "dns"
-    DATA_PROCCESING = "data_processing"
+    DATA_PROCESSING = "data_processing"
+    DATABASE = "database"
     IDENTITY = "identity"
     IMAGE = "image"
+    MESSAGING = "messaging"
     NETWORK = "network"
     OBJECT_STORAGE = "object_storage"
     ORCHESTRATION = "orchestration"
     TELEMETRY = "telemetry"
     VOLUME = "volume"
-    APPLICATION_CATALOG = "application_catalog"
 
 
 class _TempestTestsSets(utils.ImmutableMixin, utils.EnumMixin):
@@ -91,10 +92,7 @@ class _Service(utils.ImmutableMixin, utils.EnumMixin):
 
     NOVA = "nova"
     NOVA_NET = "nova-network"
-    NOVAV21 = "novav21"
-    NOVAV3 = "novav3"
     CINDER = "cinder"
-    CINDERV2 = "cinderv2"
     MANILA = "manila"
     EC2 = "ec2"
     GLANCE = "glance"
@@ -104,47 +102,53 @@ class _Service(utils.ImmutableMixin, utils.EnumMixin):
     NEUTRON = "neutron"
     DESIGNATE = "designate"
     CEILOMETER = "ceilometer"
+    MONASCA = "monasca"
     S3 = "s3"
+    SENLIN = "senlin"
     TROVE = "trove"
     SAHARA = "sahara"
     SWIFT = "swift"
     MISTRAL = "mistral"
     MURANO = "murano"
     IRONIC = "ironic"
+    GNOCCHI = "gnocchi"
+    MAGNUM = "magnum"
+    WATCHER = "watcher"
 
 
 class _ServiceType(utils.ImmutableMixin, utils.EnumMixin):
     """OpenStack services types, mapped to service names."""
 
     VOLUME = "volume"
-    VOLUMEV2 = "volumev2"
     SHARE = "share"
     EC2 = "ec2"
     IMAGE = "image"
     CLOUD = "cloudformation"
     ORCHESTRATION = "orchestration"
     IDENTITY = "identity"
+    CLUSTERING = "clustering"
     COMPUTE = "compute"
-    COMPUTEV21 = "computev21"
-    COMPUTEV3 = "computev3"
     NETWORK = "network"
     DNS = "dns"
     METERING = "metering"
+    MONITORING = "monitoring"
     S3 = "s3"
     DATABASE = "database"
-    DATA_PROCESSING = "data_processing"
+    DATA_PROCESSING = "data-processing"
+    DATA_PROCESSING_MOS = "data_processing"
     OBJECT_STORE = "object-store"
     WORKFLOW_EXECUTION = "workflowv2"
-    APPLICATION_CATALOG = "application_catalog"
+    APPLICATION_CATALOG = "application-catalog"
     BARE_METAL = "baremetal"
+    METRIC = "metric"
+    CONTAINER_INFRA = "container-infra"
+    INFRA_OPTIM = "infra-optim"
 
     def __init__(self):
         self.__names = {
+            self.CLUSTERING: _Service.SENLIN,
             self.COMPUTE: _Service.NOVA,
-            self.COMPUTEV21: _Service.NOVAV21,
-            self.COMPUTEV3: _Service.NOVAV3,
             self.VOLUME: _Service.CINDER,
-            self.VOLUMEV2: _Service.CINDERV2,
             self.SHARE: _Service.MANILA,
             self.EC2: _Service.EC2,
             self.IMAGE: _Service.GLANCE,
@@ -154,13 +158,18 @@ class _ServiceType(utils.ImmutableMixin, utils.EnumMixin):
             self.NETWORK: _Service.NEUTRON,
             self.DNS: _Service.DESIGNATE,
             self.METERING: _Service.CEILOMETER,
+            self.MONITORING: _Service.MONASCA,
             self.S3: _Service.S3,
             self.DATABASE: _Service.TROVE,
             self.DATA_PROCESSING: _Service.SAHARA,
+            self.DATA_PROCESSING_MOS: _Service.SAHARA,
             self.OBJECT_STORE: _Service.SWIFT,
             self.WORKFLOW_EXECUTION: _Service.MISTRAL,
             self.APPLICATION_CATALOG: _Service.MURANO,
             self.BARE_METAL: _Service.IRONIC,
+            self.METRIC: _Service.GNOCCHI,
+            self.CONTAINER_INFRA: _Service.MAGNUM,
+            self.INFRA_OPTIM: _Service.WATCHER,
         }
 
     def __getitem__(self, service_type):
