@@ -130,9 +130,9 @@ class ExistingCloud(engine.Engine):
                                      consts.EndpointType.PUBLIC),
             endpoint=common.get("endpoint"),
             domain_name=user.get("domain_name"),
-            user_domain_name=user.get("user_domain_name", "Default"),
+            user_domain_name=user.get("user_domain_name", None),
             admin_domain_name=user.get("admin_domain_name", "Default"),
-            project_domain_name=user.get("project_domain_name", "Default"),
+            project_domain_name=user.get("project_domain_name", None),
             https_insecure=common.get("https_insecure", False),
             https_cacert=common.get("https_cacert")
         )
