@@ -66,6 +66,7 @@ def configure(name, default_version=None, default_service_type=None,
     return wrapper
 
 
+@plugin.base()
 class OSClient(plugin.Plugin):
     def __init__(self, credential, api_info, cache_obj):
         self.credential = credential
@@ -155,11 +156,26 @@ class OSClient(plugin.Plugin):
             endpoint = endpoint or self._get_endpoint()
             kc = self.keystone()
             auth = token_endpoint.Token(endpoint, kc.auth_token)
-
+        verify = self.credential.cacert or not self.credential.insecure
         return ks_session.Session(
-            auth=auth, verify=not self.credential.insecure,
+            auth=auth, verify=verify,
             timeout=CONF.openstack_client_http_timeout)
 
+    def _get_keystoneauth_session(self):
+        from keystoneauth1 import loading
+        from keystoneauth1 import session
+        loader = loading.get_plugin_loader("password")
+        plugin = loader.load_from_options(
+            auth_url=self.credential.auth_url,
+            username=self.credential.username,
+            password=self.credential.password,
+            user_domain_name=self.credential.user_domain_name,
+            project_name=self.credential.tenant_name,
+            project_domain_name=self.credential.project_domain_name)
+        sess = session.Session(auth=plugin, verify=(
+            not self.credential.insecure))
+        return sess
+
     def _get_endpoint(self, service_type=None):
         kc = self.keystone()
         api_url = kc.service_catalog.url_for(
@@ -174,12 +190,14 @@ class OSClient(plugin.Plugin):
                        project_name_key="project_id",
                        domain_name_key="domain_name",
                        user_domain_name_key="user_domain_name",
-                       project_domain_name_key="project_domain_name"
+                       project_domain_name_key="project_domain_name",
+                       cacert_key="cacert",
                        ):
         kw = {
             user_key: self.credential.username,
             password_key: self.credential.password,
-            auth_url_key: self.credential.auth_url
+            auth_url_key: self.credential.auth_url,
+            cacert_key: self.credential.cacert,
         }
         if project_name_key:
             kw.update({project_name_key: self.credential.tenant_name})
@@ -188,9 +206,11 @@ class OSClient(plugin.Plugin):
             kw.update({
                 domain_name_key: self.credential.domain_name})
             kw.update({
-                user_domain_name_key: self.credential.user_domain_name})
+                user_domain_name_key:
+                self.credential.user_domain_name or "Default"})
             kw.update({
-                project_domain_name_key: self.credential.project_domain_name})
+                project_domain_name_key:
+                self.credential.project_domain_name or "Default"})
 
         return kw
 
@@ -313,7 +333,6 @@ class Nova(OSClient):
                              http_log_debug=logging.is_debug(),
                              timeout=CONF.openstack_client_http_timeout,
                              insecure=self.credential.insecure,
-                             cacert=self.credential.cacert,
                              **self._get_auth_info(password_key="api_key"))
         client.set_management_url(compute_api_url)
         return client
@@ -335,10 +354,9 @@ class Neutron(OSClient):
                                 endpoint_url=network_api_url,
                                 timeout=CONF.openstack_client_http_timeout,
                                 insecure=self.credential.insecure,
-                                ca_cert=self.credential.cacert,
                                 **self._get_auth_info(
-                                    project_name_key="tenant_name")
-                                )
+                                    project_name_key="tenant_name",
+                                    cacert_key="ca_cert"))
         return client
 
 
@@ -378,8 +396,8 @@ class Heat(OSClient):
                              token=kc.auth_token,
                              timeout=CONF.openstack_client_http_timeout,
                              insecure=self.credential.insecure,
-                             cacert=self.credential.cacert,
-                             **self._get_auth_info(project_name_key=None))
+                             **self._get_auth_info(project_name_key=None,
+                                                   cacert_key="ca_file"))
         return client
 
 
@@ -393,7 +411,6 @@ class Cinder(OSClient):
                                http_log_debug=logging.is_debug(),
                                timeout=CONF.openstack_client_http_timeout,
                                insecure=self.credential.insecure,
-                               cacert=self.credential.cacert,
                                **self._get_auth_info(password_key="api_key"))
         kc = self.keystone()
         volume_api_url = kc.service_catalog.url_for(
@@ -417,7 +434,6 @@ class Manila(OSClient):
             http_log_debug=logging.is_debug(),
             timeout=CONF.openstack_client_http_timeout,
             insecure=self.credential.insecure,
-            cacert=self.credential.cacert,
             **self._get_auth_info(password_key="api_key",
                                   project_name_key="project_name"))
         kc = self.keystone()
@@ -451,11 +467,26 @@ class Ceilometer(OSClient):
             token=auth_token,
             timeout=CONF.openstack_client_http_timeout,
             insecure=self.credential.insecure,
-            cacert=self.credential.cacert,
             **self._get_auth_info(project_name_key="tenant_name"))
         return client
 
 
+@configure("gnocchi", default_service_type="metric", default_version="1",
+           supported_versions=["1"])
+class Gnocchi(OSClient):
+
+    def create_client(self, version=None, service_type=None):
+        """Return gnocchi client."""
+        # NOTE(sumantmurke): gnocchiclient requires keystoneauth1 for
+        # authenticating and creating a session.
+        from gnocchiclient import client as gnocchi
+        service_type = self.choose_service_type(service_type)
+        sess = self._get_keystoneauth_session()
+        gclient = gnocchi.Client(version=self.choose_version(
+            version), session=sess, service_type=service_type)
+        return gclient
+
+
 @configure("ironic", default_version="1", default_service_type="baremetal",
            supported_versions=["1"])
 class Ironic(OSClient):
@@ -497,6 +528,7 @@ class Sahara(OSClient):
         client = sahara.Client(
             self.choose_version(version),
             service_type=self.choose_service_type(service_type),
+            endpoint_type=self.credential.endpoint_type,
             insecure=self.credential.insecure,
             **self._get_auth_info(password_key="api_key",
                                   project_name_key="project_name"))
@@ -579,7 +611,6 @@ class Trove(OSClient):
                               region_name=self.credential.region_name,
                               timeout=CONF.openstack_client_http_timeout,
                               insecure=self.credential.insecure,
-                              cacert=self.credential.cacert,
                               **self._get_auth_info(password_key="api_key")
                               )
         return client
@@ -619,11 +650,8 @@ class Swift(OSClient):
                                   preauthtoken=kc.auth_token,
                                   insecure=self.credential.insecure,
                                   cacert=self.credential.cacert,
-                                  **self._get_auth_info(
-                                      user_key="user",
-                                      password_key="key",
-                                      auth_url_key="authurl",
-                                      project_name_key="tenant_name")
+                                  user=self.credential.username,
+                                  tenant_name=self.credential.tenant_name,
                                   )
         return client
 
@@ -670,7 +698,6 @@ class Monasca(OSClient):
             token=auth_token,
             timeout=CONF.openstack_client_http_timeout,
             insecure=self.credential.insecure,
-            cacert=self.credential.cacert,
             **self._get_auth_info(project_name_key="tenant_name"))
         return client
 
@@ -691,6 +718,51 @@ class Cue(OSClient):
         return cue.Client(session=session, interface=endpoint_type[0])
 
 
+@configure("senlin", default_version="1", default_service_type="clustering",
+           supported_versions=["1"])
+class Senlin(OSClient):
+    def create_client(self, version=None, service_type=None):
+        """Return senlin client."""
+        from senlinclient import client as senlin
+        return senlin.Client(
+            self.choose_version(version),
+            **self._get_auth_info(project_name_key="project_name",
+                                  cacert_key="cert"))
+
+
+@configure("magnum", default_version="1",
+           default_service_type="container-infra",)
+class Magnum(OSClient):
+    def create_client(self, version=None, service_type=None):
+        """Return magnum client."""
+        from magnumclient import client as magnum
+
+        api_url = self._get_endpoint(service_type)
+        session = self._get_session(endpoint=api_url)
+        endpoint_type = self.credential.endpoint_type,
+
+        return magnum.Client(session=session, interface=endpoint_type[0])
+
+
+@configure("watcher", default_version="1", default_service_type="infra-optim",
+           supported_versions=["1"])
+class Watcher(OSClient):
+    def create_client(self, version=None, service_type=None):
+        """Return watcher client."""
+        from watcherclient import client as watcher_client
+        kc = self.keystone()
+        watcher_api_url = self._get_endpoint(
+            self.choose_service_type(service_type))
+        client = watcher_client.Client(
+            self.choose_version(version),
+            watcher_api_url,
+            token=kc.auth_token,
+            timeout=CONF.openstack_client_http_timeout,
+            insecure=self.credential.insecure,
+            ca_file=self.credential.cacert)
+        return client
+
+
 class Clients(object):
     """This class simplify and unify work with OpenStack python clients."""
 
@@ -713,6 +785,9 @@ class Clients(object):
                 creds["admin"]["username"],
                 creds["admin"]["password"],
                 creds["admin"]["tenant_name"],
+                endpoint_type=creds["endpoint_type"],
+                user_domain_name=creds["admin"].get("user_domain_name"),
+                project_domain_name=creds["admin"].get("project_domain_name"),
                 endpoint=creds["endpoint"],
                 region_name=creds["region_name"],
                 https_cacert=creds["https_cacert"],
