@@ -15,6 +15,8 @@
 
 from boto import exception as boto_exception
 from neutronclient.common import exceptions as neutron_exceptions
+from novaclient import exceptions as nova_exc
+from oslo_config import cfg
 from saharaclient.api import base as saharaclient_base
 
 from rally.common import logging
@@ -26,6 +28,15 @@ from rally.plugins.openstack.scenarios.keystone import utils as kutils
 from rally.plugins.openstack.scenarios.nova import utils as nova_utils
 from rally.plugins.openstack.wrappers import glance as glance_wrapper
 from rally.plugins.openstack.wrappers import keystone as keystone_wrapper
+from rally.task import utils as task_utils
+
+CONF = cfg.CONF
+CONF.import_opt("glance_image_delete_timeout",
+                "rally.plugins.openstack.scenarios.glance.utils",
+                "benchmark")
+CONF.import_opt("glance_image_delete_poll_interval",
+                "rally.plugins.openstack.scenarios.glance.utils",
+                "benchmark")
 
 LOG = logging.getLogger(__name__)
 
@@ -68,7 +79,8 @@ class HeatStack(base.ResourceManager):
 _nova_order = get_order(200)
 
 
-@base.resource("nova", "servers", order=next(_nova_order))
+@base.resource("nova", "servers", order=next(_nova_order),
+               tenant_resource=True)
 class NovaServer(base.ResourceManager):
     def list(self):
         """List all servers."""
@@ -113,7 +125,8 @@ class NovaKeypair(SynchronizedDeletion, base.ResourceManager):
     pass
 
 
-@base.resource("nova", "security_groups", order=next(_nova_order))
+@base.resource("nova", "security_groups", order=next(_nova_order),
+               tenant_resource=True)
 class NovaSecurityGroup(SynchronizedDeletion, base.ResourceManager):
 
     def list(self):
@@ -127,6 +140,22 @@ class NovaQuotas(QuotaMixin, base.ResourceManager):
     pass
 
 
+@base.resource("nova", "flavors", order=next(_nova_order),
+               admin_required=True, perform_for_admin_only=True)
+class NovaFlavors(base.ResourceManager):
+    def list(self):
+        return [r for r in self._manager().list()
+                if utils.name_matches_object(r.name, nova_utils.NovaScenario)]
+
+    def is_deleted(self):
+        try:
+            self._manager().get(self.name())
+        except nova_exc.NotFound:
+            return True
+
+        return False
+
+
 @base.resource("nova", "floating_ips_bulk", order=next(_nova_order),
                admin_required=True)
 class NovaFloatingIpsBulk(SynchronizedDeletion, base.ResourceManager):
@@ -390,15 +419,24 @@ class ManilaSecurityService(base.ResourceManager):
 @base.resource("glance", "images", order=500, tenant_resource=True)
 class GlanceImage(base.ResourceManager):
 
+    def _client(self):
+        return getattr(self.admin or self.user, self._service)
+
     def _wrapper(self):
-        return glance_wrapper.wrap(
-            getattr(self.admin or self.user, self._service), self)
+        return glance_wrapper.wrap(self._client(), self)
 
     def list(self):
         return self._wrapper().list_images(owner=self.tenant_uuid)
 
     def delete(self):
-        return self._wrapper().delete_image(self.raw_resource)
+        client = self._client()
+        client().images.delete(self.raw_resource.id)
+        task_utils.wait_for_status(
+            self.raw_resource, ["deleted"],
+            check_deletion=True,
+            update_resource=self._wrapper().get_image,
+            timeout=CONF.benchmark.glance_image_delete_timeout,
+            check_interval=CONF.benchmark.glance_image_delete_poll_interval)
 
 
 # SAHARA
@@ -682,6 +720,27 @@ class FuelEnvironment(base.ResourceManager):
                                              futils.FuelScenario)]
 
 
+# WATCHER
+
+@base.resource("watcher", "audit_template", order=1500,
+               admin_required=True, tenant_resource=True)
+class WatcherTemplate(SynchronizedDeletion, base.ResourceManager):
+
+    def id(self):
+        return self.raw_resource.uuid
+
+    def is_deleted(self):
+        from watcherclient.common.apiclient import exceptions
+        try:
+            self._manager().get(self.id())
+            return False
+        except exceptions.NotFound:
+            return True
+
+    def list(self):
+        return self._manager().list(limit=0)
+
+
 # KEYSTONE
 
 _keystone_order = get_order(9000)
@@ -700,8 +759,8 @@ class KeystoneMixin(SynchronizedDeletion):
         # TODO(boris-42): We should use such stuff in all list commands.
         resources = self._resource + "s"
         list_method = getattr(self._manager(), "list_%s" % resources)
-
-        return filter(kutils.is_temporary, list_method())
+        return [r for r in list_method()
+                if utils.name_matches_object(r.name, kutils.KeystoneScenario)]
 
 
 @base.resource("keystone", "user", order=next(_keystone_order),
