@@ -22,11 +22,10 @@ from rally.plugins.openstack import scenario
 from rally.plugins.openstack.scenarios.cinder import utils as cinder_utils
 from rally.plugins.openstack.scenarios.glance import utils as glance_utils
 from rally.plugins.openstack.scenarios.nova import utils as nova_utils
+from rally.task import atomic
 from rally.task import types
 from rally.task import validation
 
-LOG = logging.getLogger(__name__)
-
 
 class CinderVolumes(cinder_utils.CinderScenario,
                     nova_utils.NovaScenario,
@@ -364,9 +363,14 @@ class CinderVolumes(cinder_utils.CinderScenario,
     @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
     @validation.required_openstack(users=True)
     @scenario.configure(context={"cleanup": ["cinder", "nova"]})
+    @logging.log_deprecated_args(
+        "Use 'create_snapshot_kwargs' for additional snapshot kwargs.",
+        "0.4.1", ["kwargs"], once=True)
     def create_nested_snapshots_and_attach_volume(self,
                                                   size=None,
                                                   nested_level=1,
+                                                  create_volume_kwargs=None,
+                                                  create_snapshot_kwargs=None,
                                                   **kwargs):
 
         """Create a volume from snapshot and attach/detach the volume
@@ -381,6 +385,8 @@ class CinderVolumes(cinder_utils.CinderScenario,
                         max - maximum size volumes will be created as.
                      default values: {"min": 1, "max": 5}
         :param nested_level: amount of nested levels
+        :param create_volume_kwargs: optional args to create a volume
+        :param create_snapshot_kwargs: optional args to create a snapshot
         :param kwargs: Optional parameters used during volume
                        snapshot creation.
         """
@@ -393,16 +399,22 @@ class CinderVolumes(cinder_utils.CinderScenario,
         #       size in _create_volume method.
         size = random.randint(size["min"], size["max"])
 
-        source_vol = self._create_volume(size)
+        create_volume_kwargs = create_volume_kwargs or {}
+
+        create_snapshot_kwargs = create_snapshot_kwargs or kwargs or {}
+
+        source_vol = self._create_volume(size, **create_volume_kwargs)
         nes_objs = [(self.get_random_server(), source_vol,
-                     self._create_snapshot(source_vol.id, False, **kwargs))]
+                     self._create_snapshot(source_vol.id, False,
+                                           **create_snapshot_kwargs))]
 
         self._attach_volume(nes_objs[0][0], nes_objs[0][1])
         snapshot = nes_objs[0][2]
 
         for i in range(nested_level - 1):
             volume = self._create_volume(size, snapshot_id=snapshot.id)
-            snapshot = self._create_snapshot(volume.id, False, **kwargs)
+            snapshot = self._create_snapshot(volume.id, False,
+                                             **create_snapshot_kwargs)
             server = self.get_random_server()
             self._attach_volume(server, volume)
 
@@ -432,11 +444,12 @@ class CinderVolumes(cinder_utils.CinderScenario,
         self._create_snapshot(volume["id"], force=force, **kwargs)
         self._list_snapshots(detailed)
 
+    @types.convert(image={"type": "glance_image"})
     @validation.required_services(consts.Service.CINDER, consts.Service.GLANCE)
     @validation.required_openstack(users=True)
     @validation.required_parameters("size")
     @scenario.configure(context={"cleanup": ["cinder", "glance"]})
-    def create_and_upload_volume_to_image(self, size, force=False,
+    def create_and_upload_volume_to_image(self, size, image=None, force=False,
                                           container_format="bare",
                                           disk_format="raw",
                                           do_delete=True,
@@ -447,6 +460,7 @@ class CinderVolumes(cinder_utils.CinderScenario,
                      dictionary, must contain two values:
                          min - minimum size volumes will be created as;
                          max - maximum size volumes will be created as.
+        :param image: image to be used to create volume.
         :param force: when set to True volume that is attached to an instance
                       could be uploaded to image
         :param container_format: image container format
@@ -454,6 +468,8 @@ class CinderVolumes(cinder_utils.CinderScenario,
         :param do_delete: deletes image and volume after uploading if True
         :param kwargs: optional args to create a volume
         """
+        if image:
+            kwargs["imageRef"] = image
         volume = self._create_volume(size, **kwargs)
         image = self._upload_volume_to_image(volume, force, container_format,
                                              disk_format)
@@ -540,3 +556,67 @@ class CinderVolumes(cinder_utils.CinderScenario,
         if do_delete:
             self._delete_volume(volume)
             self._delete_backup(backup)
+
+    @types.convert(image={"type": "glance_image"})
+    @validation.image_exists("image", nullable=True)
+    @validation.required_services(consts.Service.CINDER)
+    @validation.required_openstack(users=True)
+    @scenario.configure(context={"cleanup": ["cinder"]})
+    def create_volume_and_clone(self, size, image=None, nested_level=1,
+                                **kwargs):
+        """Create a volume, then clone it to another volume.
+
+          This creates a volume, then clone it to anothor volume,
+          and then clone the new volume to next volume...
+           1. create source volume (from image)
+           2. clone source volume to volume1
+           3. clone volume1 to volume2
+           4. clone volume2 to volume3
+           5. ...
+
+        :param size: volume size (integer, in GB) or
+                     dictionary, must contain two values:
+                         min - minimum size volumes will be created as;
+                         max - maximum size volumes will be created as.
+        :param image: image to be used to create initial volume
+        :param nested_level: amount of nested levels
+        :param kwargs: optional args to create volumes
+        """
+        if image:
+            kwargs["imageRef"] = image
+
+        source_vol = self._create_volume(size, **kwargs)
+
+        kwargs.pop("imageRef", None)
+        for i in range(nested_level):
+            with atomic.ActionTimer(self, "cinder.clone_volume"):
+                source_vol = self._create_volume(source_vol.size,
+                                                 source_volid=source_vol.id,
+                                                 atomic_action=False, **kwargs)
+
+    @validation.required_services(consts.Service.CINDER)
+    @validation.required_contexts("volumes")
+    @validation.required_openstack(users=True)
+    @scenario.configure(context={"cleanup": ["cinder"]})
+    def create_volume_from_snapshot(self, do_delete=True,
+                                    create_snapshot_kwargs=None,
+                                    **kwargs):
+        """Create a volume-snapshot, then create a volume from this snapshot.
+
+        :param do_delete: if True, a snapshot and a volume will
+                          be deleted after creation.
+        :param create_snapshot_kwargs: optional args to create a snapshot
+        :param kwargs: optional args to create a volume
+        """
+        create_snapshot_kwargs = create_snapshot_kwargs or {}
+        src_volume = random.choice(self.context["tenant"]["volumes"])
+
+        snapshot = self._create_snapshot(src_volume["id"],
+                                         **create_snapshot_kwargs)
+        volume = self._create_volume(src_volume["size"],
+                                     snapshot_id=snapshot.id,
+                                     **kwargs)
+
+        if do_delete:
+            self._delete_snapshot(snapshot)
+            self._delete_volume(volume)
