@@ -129,12 +129,13 @@ class NovaServers(utils.NovaScenario,
 
     @types.convert(image={"type": "glance_image"},
                    flavor={"type": "nova_flavor"})
-    @validation.image_valid_on_flavor("flavor", "image")
+    @validation.image_valid_on_flavor("flavor", "image", validate_disk=False)
     @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
     @validation.required_openstack(users=True)
     @scenario.configure(context={"cleanup": ["nova", "cinder"]})
     def boot_server_from_volume_and_delete(self, image, flavor,
                                            volume_size,
+                                           volume_type=None,
                                            min_sleep=0, max_sleep=0,
                                            force_delete=False, **kwargs):
         """Boot a server from volume and then delete it.
@@ -147,14 +148,17 @@ class NovaServers(utils.NovaScenario,
         :param image: image to be used to boot an instance
         :param flavor: flavor to be used to boot an instance
         :param volume_size: volume size (in GB)
+        :param volume_type: specifies volume type when there are
+                            multiple backends
         :param min_sleep: Minimum sleep time in seconds (non-negative)
         :param max_sleep: Maximum sleep time in seconds (non-negative)
         :param force_delete: True if force_delete should be used
         :param kwargs: Optional additional arguments for server creation
         """
-        volume = self._create_volume(volume_size, imageRef=image)
+        volume = self._create_volume(volume_size, imageRef=image,
+                                     volume_type=volume_type)
         block_device_mapping = {"vda": "%s:::1" % volume.id}
-        server = self._boot_server(image, flavor,
+        server = self._boot_server(None, flavor,
                                    block_device_mapping=block_device_mapping,
                                    **kwargs)
         self.sleep_between(min_sleep, max_sleep)
@@ -273,12 +277,13 @@ class NovaServers(utils.NovaScenario,
 
     @types.convert(image={"type": "glance_image"},
                    flavor={"type": "nova_flavor"})
-    @validation.image_valid_on_flavor("flavor", "image")
+    @validation.image_valid_on_flavor("flavor", "image", validate_disk=False)
     @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
     @validation.required_openstack(users=True)
     @scenario.configure(context={"cleanup": ["nova", "cinder"]})
     def boot_server_from_volume(self, image, flavor, volume_size,
-                                auto_assign_nic=False, **kwargs):
+                                volume_type=None, auto_assign_nic=False,
+                                **kwargs):
         """Boot a server from volume.
 
         The scenario first creates a volume and then a server.
@@ -287,12 +292,15 @@ class NovaServers(utils.NovaScenario,
         :param image: image to be used to boot an instance
         :param flavor: flavor to be used to boot an instance
         :param volume_size: volume size (in GB)
+        :param volume_type: specifies volume type when there are
+                            multiple backends
         :param auto_assign_nic: True if NICs should be assigned
         :param kwargs: Optional additional arguments for server creation
         """
-        volume = self._create_volume(volume_size, imageRef=image)
+        volume = self._create_volume(volume_size, imageRef=image,
+                                     volume_type=volume_type)
         block_device_mapping = {"vda": "%s:::1" % volume.id}
-        self._boot_server(image, flavor, auto_assign_nic=auto_assign_nic,
+        self._boot_server(None, flavor, auto_assign_nic=auto_assign_nic,
                           block_device_mapping=block_device_mapping,
                           **kwargs)
 
@@ -489,7 +497,7 @@ class NovaServers(utils.NovaScenario,
     @types.convert(image={"type": "glance_image"},
                    flavor={"type": "nova_flavor"},
                    to_flavor={"type": "nova_flavor"})
-    @validation.image_valid_on_flavor("flavor", "image")
+    @validation.image_valid_on_flavor("flavor", "image", validate_disk=False)
     @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
     @validation.required_openstack(users=True)
     @scenario.configure(context={"cleanup": ["nova", "cinder"]})
@@ -531,7 +539,7 @@ class NovaServers(utils.NovaScenario,
         boot_server_kwargs["block_device_mapping"] = {
             "vda": "%s:::1" % volume.id}
 
-        server = self._boot_server(image, flavor, **boot_server_kwargs)
+        server = self._boot_server(None, flavor, **boot_server_kwargs)
         self.sleep_between(min_sleep, max_sleep)
         self._resize(server, to_flavor)
 
@@ -643,12 +651,13 @@ class NovaServers(utils.NovaScenario,
 
     @types.convert(image={"type": "glance_image"},
                    flavor={"type": "nova_flavor"})
-    @validation.image_valid_on_flavor("flavor", "image")
+    @validation.image_valid_on_flavor("flavor", "image", validate_disk=False)
     @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
     @validation.required_openstack(admin=True, users=True)
     @scenario.configure(context={"cleanup": ["nova", "cinder"]})
     def boot_server_from_volume_and_live_migrate(self, image, flavor,
                                                  volume_size,
+                                                 volume_type=None,
                                                  block_migration=False,
                                                  disk_over_commit=False,
                                                  force_delete=False,
@@ -668,6 +677,8 @@ class NovaServers(utils.NovaScenario,
         :param image: image to be used to boot an instance
         :param flavor: flavor to be used to boot an instance
         :param volume_size: volume size (in GB)
+        :param volume_type: specifies volume type when there are
+                            multiple backends
         :param block_migration: Specifies the migration type
         :param disk_over_commit: Specifies whether to allow overcommit
                                  on migrated instance or not
@@ -676,9 +687,10 @@ class NovaServers(utils.NovaScenario,
         :param max_sleep: Maximum sleep time in seconds (non-negative)
         :param kwargs: Optional additional arguments for server creation
         """
-        volume = self._create_volume(volume_size, imageRef=image)
+        volume = self._create_volume(volume_size, imageRef=image,
+                                     volume_type=volume_type)
         block_device_mapping = {"vda": "%s:::1" % volume.id}
-        server = self._boot_server(image, flavor,
+        server = self._boot_server(None, flavor,
                                    block_device_mapping=block_device_mapping,
                                    **kwargs)
         self.sleep_between(min_sleep, max_sleep)
@@ -758,7 +770,7 @@ class NovaServers(utils.NovaScenario,
         """Migrate a server.
 
         This scenario launches a VM on a compute node available in
-        the availability zone and stops the VM, and then migrates the VM
+        the availability zone, and then migrates the VM
         to another compute node on the same availability zone.
 
         :param image: image to be used to boot an instance
@@ -766,15 +778,14 @@ class NovaServers(utils.NovaScenario,
         :param kwargs: Optional additional arguments for server creation
         """
         server = self._boot_server(image, flavor, **kwargs)
-        self._stop_server(server)
         self._migrate(server)
         # NOTE(wtakase): This is required because cold migration and resize
         #                share same code path.
         confirm = kwargs.get("confirm", True)
         if confirm:
-            self._resize_confirm(server, status="SHUTOFF")
+            self._resize_confirm(server, status="ACTIVE")
         else:
-            self._resize_revert(server, status="SHUTOFF")
+            self._resize_revert(server, status="ACTIVE")
         self._delete_server(server)
 
     @types.convert(from_image={"type": "glance_image"},
@@ -861,3 +872,56 @@ class NovaServers(utils.NovaScenario,
         """
         server = self._boot_server(image, flavor, **kwargs)
         self._get_server_console_output(server, length)
+
+    @types.convert(image={"type": "glance_image"},
+                   flavor={"type": "nova_flavor"})
+    @validation.image_valid_on_flavor("flavor", "image")
+    @validation.required_services(consts.Service.NOVA)
+    @validation.required_openstack(users=True)
+    @scenario.configure(context={"cleanup": ["nova"]})
+    def boot_and_update_server(self, image, flavor, description=None,
+                               **kwargs):
+        """Boot a server, then update its name and description.
+
+        The scenario first creates a server, then update it.
+        Assumes that cleanup is done elsewhere.
+
+        :param image: image to be used to boot an instance
+        :param flavor: flavor to be used to boot an instance
+        :param description: update the server description
+        :param kwargs: Optional additional arguments for server creation
+        """
+        server = self._boot_server(image, flavor, **kwargs)
+        self._update_server(server, description)
+
+    @types.convert(image={"type": "glance_image"},
+                   flavor={"type": "nova_flavor"})
+    @validation.image_valid_on_flavor("flavor", "image")
+    @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
+    @validation.required_openstack(users=True)
+    @scenario.configure(context={"cleanup": ["nova", "cinder"]})
+    def boot_server_from_volume_snapshot(self, image, flavor, volume_size,
+                                         volume_type=None,
+                                         auto_assign_nic=False, **kwargs):
+        """Boot a server from a snapshot.
+
+        The scenario first creates a volume and creates a
+        snapshot from this volume, then boots a server from
+        the created snapshot.
+        Assumes that cleanup is done elsewhere.
+
+        :param image: image to be used to boot an instance
+        :param flavor: flavor to be used to boot an instance
+        :param volume_size: volume size (in GB)
+        :param volume_type: specifies volume type when there are
+                            multiple backends
+        :param auto_assign_nic: True if NICs should be assigned
+        :param kwargs: Optional additional arguments for server creation
+        """
+        volume = self._create_volume(volume_size, imageRef=image,
+                                     volume_type=volume_type)
+        snapshot = self._create_snapshot(volume.id, False)
+        block_device_mapping = {"vda": "%s:snap::1" % snapshot.id}
+        self._boot_server(None, flavor, auto_assign_nic=auto_assign_nic,
+                          block_device_mapping=block_device_mapping,
+                          **kwargs)
