@@ -21,6 +21,7 @@ import six
 
 from rally import exceptions
 from rally.plugins.openstack import scenario
+from rally.plugins.openstack.wrappers import glance as glance_wrapper
 from rally.plugins.openstack.wrappers import network as network_wrapper
 from rally.task import atomic
 from rally.task import utils
@@ -464,13 +465,14 @@ class NovaScenario(scenario.OpenStackScenario):
 
         :param image: Image object
         """
-        image.delete()
+        self.clients("glance").images.delete(image.id)
+        wrapper = glance_wrapper.wrap(self._clients.glance, self)
         check_interval = CONF.benchmark.nova_server_image_delete_poll_interval
         utils.wait_for_status(
             image,
-            ready_statuses=["deleted"],
+            ready_statuses=["deleted", "pending_delete"],
             check_deletion=True,
-            update_resource=utils.get_from_manager(),
+            update_resource=wrapper.get_image,
             timeout=CONF.benchmark.nova_server_image_delete_timeout,
             check_interval=check_interval
         )
@@ -750,7 +752,7 @@ class NovaScenario(scenario.OpenStackScenario):
             new_host = random.choice(
                 [key for key, value in six.iteritems(az.hosts)
                     if key != host and
-                    value["nova-compute"]["available"] is True])
+                    value.get("nova-compute", {}).get("available", False)])
             return new_host
         except IndexError:
             raise exceptions.InvalidHostException(
@@ -903,3 +905,82 @@ class NovaScenario(scenario.OpenStackScenario):
         :returns: flavors list
         """
         return self.clients("nova").flavors.list(detailed, **kwargs)
+
+    @atomic.action_timer("nova.list_agents")
+    def _list_agents(self, hypervisor=None):
+        """List all nova-agent builds.
+
+        :param hypervisor: The nova-hypervisor ID on which we need to list all
+                           the builds
+        :returns: Nova-agent build list
+        """
+        return self.admin_clients("nova").agents.list(hypervisor)
+
+    @atomic.action_timer("nova.list_aggregates")
+    def _list_aggregates(self):
+        """Returns list of all os-aggregates."""
+        return self.admin_clients("nova").aggregates.list()
+
+    @atomic.action_timer("nova.list_availbility_zones")
+    def _list_availability_zones(self, detailed=True):
+        """List availability-zones.
+
+        :param detailed: True if the availability-zone listing should contain
+                         detailed information
+        :returns: Availability-zone list
+        """
+        return self.admin_clients("nova").availability_zones.list(detailed)
+
+    @atomic.action_timer("nova.list_hosts")
+    def _list_hosts(self, zone=None):
+        """List nova hosts.
+
+        :param zone: List all hosts in the given nova availability-zone ID
+        :returns: Nova host list
+        """
+        return self.admin_clients("nova").hosts.list(zone)
+
+    @atomic.action_timer("nova.list_services")
+    def _list_services(self, host=None, binary=None):
+        """return all nova service details
+
+        :param host: List all nova services on host
+        :param binary: List all nova services matching  given binary
+        """
+        return self.admin_clients("nova").services.list(host, binary)
+
+    @atomic.action_timer("nova.create_flavor")
+    def _create_flavor(self, ram, vcpus, disk, **kwargs):
+        """Create a flavor
+
+        :param ram: Memory in MB for the flavor
+        :param vcpus: Number of VCPUs for the flavor
+        :param disk: Size of local disk in GB
+        :param kwargs: Optional additional arguments for flavor creation
+        """
+        name = self.generate_random_name()
+        return self.admin_clients("nova").flavors.create(name, ram, vcpus,
+                                                         disk, **kwargs)
+
+    @atomic.action_timer("nova.list_flavor_access")
+    def _list_flavor_access(self, flavor):
+        """List access-rules for non-public flavor.
+
+        :param flavor: List access rules for flavor instance or flavor ID
+        """
+        return self.admin_clients("nova").flavor_access.list(flavor=flavor)
+
+    @atomic.action_timer("nova.update_server")
+    def _update_server(self, server, description=None):
+        """update the server's name and description.
+
+        :param server: Server object
+        :param description: update the server description
+        :returns: The updated server
+        """
+        new_name = self.generate_random_name()
+        if description:
+            return server.update(name=new_name,
+                                 description=description)
+        else:
+            return server.update(name=new_name)
