@@ -32,9 +32,6 @@ from rally.task import validation
 
 LOG = logging.getLogger(__name__)
 
-ICMP_UP_STATUS = "ICMP UP"
-ICMP_DOWN_STATUS = "ICMP DOWN"
-
 VM_BENCHMARK_OPTS = [
     cfg.FloatOpt("vm_ping_poll_interval", default=1.0,
                  help="Interval between checks when waiting for a VM to "
@@ -47,6 +44,52 @@ benchmark_group = cfg.OptGroup(name="benchmark", title="benchmark options")
 CONF.register_opts(VM_BENCHMARK_OPTS, group=benchmark_group)
 
 
+class Host(object):
+
+    ICMP_UP_STATUS = "ICMP UP"
+    ICMP_DOWN_STATUS = "ICMP DOWN"
+
+    name = "ip"
+
+    def __init__(self, ip):
+        self.ip = netaddr.IPAddress(ip)
+        self.status = self.ICMP_DOWN_STATUS
+
+    @property
+    def id(self):
+        return self.ip.format()
+
+    @classmethod
+    def update_status(cls, server):
+        """Check ip address is pingable and update status."""
+        ping = "ping" if server.ip.version == 4 else "ping6"
+        if sys.platform.startswith("linux"):
+            cmd = [ping, "-c1", "-w1", server.ip.format()]
+        else:
+            cmd = [ping, "-c1", server.ip.format()]
+
+        proc = subprocess.Popen(cmd,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE)
+        proc.wait()
+        LOG.debug("Host %s is ICMP %s"
+                  % (server.ip.format(), proc.returncode and "down" or "up"))
+        if proc.returncode == 0:
+            server.status = cls.ICMP_UP_STATUS
+        else:
+            server.status = cls.ICMP_DOWN_STATUS
+        return server
+
+    def __eq__(self, other):
+        if not isinstance(other, Host):
+            raise TypeError("%s should be an instance of %s" % (
+                other, Host.__class__.__name__))
+        return self.ip == other.ip and self.status == other.status
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+
 class VMScenario(nova_utils.NovaScenario, cinder_utils.CinderScenario):
     """Base class for VM scenarios with basic atomic actions.
 
@@ -153,21 +196,22 @@ class VMScenario(nova_utils.NovaScenario, cinder_utils.CinderScenario):
         return self._delete_server(server, force=force_delete)
 
     @atomic.action_timer("vm.wait_for_ssh")
-    def _wait_for_ssh(self, ssh):
-        ssh.wait()
+    def _wait_for_ssh(self, ssh, timeout=120, interval=1):
+        ssh.wait(timeout, interval)
 
     @atomic.action_timer("vm.wait_for_ping")
     def _wait_for_ping(self, server_ip):
-        server_ip = netaddr.IPAddress(server_ip)
-        utils.wait_for(
-            server_ip,
-            is_ready=utils.resource_is(ICMP_UP_STATUS, self._ping_ip_address),
+        server = Host(server_ip)
+        utils.wait_for_status(
+            server,
+            ready_statuses=[Host.ICMP_UP_STATUS],
+            update_resource=Host.update_status,
             timeout=CONF.benchmark.vm_ping_timeout,
             check_interval=CONF.benchmark.vm_ping_poll_interval
         )
 
     def _run_command(self, server_ip, port, username, password, command,
-                     pkey=None):
+                     pkey=None, timeout=120, interval=1):
         """Run command via SSH on server.
 
         Create SSH connection for server, wait for server to become available
@@ -183,31 +227,13 @@ class VMScenario(nova_utils.NovaScenario, cinder_utils.CinderScenario):
             See `rally info find VMTasks.boot_runcommand_delete' parameter
             `command' docstring for explanation.
         :param pkey: key for SSH authentication
+        :param timeout: wait for ssh timeout. Default is 120 seconds
+        :param interval: ssh retry interval. Default is 1 second
 
         :returns: tuple (exit_status, stdout, stderr)
         """
         pkey = pkey if pkey else self.context["user"]["keypair"]["private"]
         ssh = sshutils.SSH(username, server_ip, port=port,
                            pkey=pkey, password=password)
-        self._wait_for_ssh(ssh)
+        self._wait_for_ssh(ssh, timeout, interval)
         return self._run_command_over_ssh(ssh, command)
-
-    @staticmethod
-    def _ping_ip_address(host):
-        """Check ip address that it is pingable.
-
-        :param host: instance of `netaddr.IPAddress`
-        """
-        ping = "ping" if host.version == 4 else "ping6"
-        if sys.platform.startswith("linux"):
-            cmd = [ping, "-c1", "-w1", str(host)]
-        else:
-            cmd = [ping, "-c1", str(host)]
-
-        proc = subprocess.Popen(cmd,
-                                stdout=subprocess.PIPE,
-                                stderr=subprocess.PIPE)
-        proc.wait()
-        LOG.debug("Host %s is ICMP %s"
-                  % (host, proc.returncode and "down" or "up"))
-        return ICMP_UP_STATUS if (proc.returncode == 0) else ICMP_DOWN_STATUS
