@@ -110,14 +110,15 @@ class ResultConsumer(object):
         # NOTE(boris-42): Sort in order of starting instead of order of ending
         self.results.sort(key=lambda x: x["timestamp"])
 
-        LOG.info("Load duration is: %s" %
-                 (self.load_finished_at - self.load_started_at))
+        load_duration = max(self.load_finished_at - self.load_started_at, 0)
+
+        LOG.info("Load duration is: %s" % load_duration)
         LOG.info("Full runner duration is: %s" % self.runner.run_duration)
         LOG.info("Full duration is %s" % (self.finish - self.start))
 
         self.task.append_results(self.key, {
             "raw": self.results,
-            "load_duration": self.load_finished_at - self.load_started_at,
+            "load_duration": load_duration,
             "full_duration": self.finish - self.start,
             "sla": self.sla_checker.results()})
 
@@ -185,11 +186,6 @@ class TaskEngine(object):
         :param abort_on_sla_failure: True if the execution should be stopped
                                      when some SLA check fails
         """
-        if config is None:
-            msg = _("Input task is empty")
-            task.set_failed(log=msg)
-            raise exceptions.InvalidTaskException(msg)
-
         try:
             self.config = TaskConfig(config)
         except Exception as e:
@@ -456,6 +452,13 @@ class TaskConfig(object):
 
         :param config: Dict with configuration of specified task
         """
+        if config is None:
+            # NOTE(stpierre): This gets reraised as
+            # InvalidTaskException. if we raise it here as
+            # InvalidTaskException, then "Task config is invalid: "
+            # gets prepended to the message twice.
+            raise Exception(_("Input task is empty"))
+
         self.version = self._get_version(config)
         self._validate_version()
         self._validate_json(config)
