@@ -30,9 +30,9 @@ def configure(name, namespace="default"):
     return plugin.configure(name=name, namespace=namespace)
 
 
+@plugin.base()
 @six.add_metaclass(abc.ABCMeta)
-@configure(name="base-exporter")
-class TaskExporter(plugin.Plugin):
+class Exporter(plugin.Plugin):
 
     def __init__(self, connection_string):
         self.connection_string = connection_string
@@ -46,4 +46,6 @@ class TaskExporter(plugin.Plugin):
 
     @abc.abstractmethod
     def validate(self):
-        """Used to validate connection string."""
\ No newline at end of file
+        """Used to validate connection string."""
+
+TaskExporter = Exporter
