@@ -28,6 +28,7 @@ from rally import consts
 from rally import exceptions
 from rally import osclients
 from rally.plugins.openstack.context.nova import flavors as flavors_ctx
+from rally.plugins.openstack import types as openstack_types
 from rally.task import types
 from rally.verification.tempest import tempest
 
@@ -253,7 +254,7 @@ def _get_validated_image(config, clients, param_name):
             }
             return (ValidationResult(True), image)
     try:
-        image_id = types.ImageResourceType.transform(
+        image_id = openstack_types.GlanceImage.transform(
             clients=clients, resource_config=image_args)
         image = clients.glance().images.get(image=image_id).to_dict()
         if not image.get("size"):
@@ -287,7 +288,7 @@ def _get_validated_flavor(config, clients, param_name):
         msg = "Parameter %s is not specified." % param_name
         return (ValidationResult(False, msg), None)
     try:
-        flavor_id = types.FlavorResourceType.transform(
+        flavor_id = openstack_types.Flavor.transform(
             clients=clients, resource_config=flavor_value)
         flavor = clients.nova().flavors.get(flavor=flavor_id)
         return (ValidationResult(True), flavor)
@@ -338,13 +339,17 @@ def flavor_exists(config, clients, deployment, param_name):
 
 @validator
 def image_valid_on_flavor(config, clients, deployment, flavor_name,
-                          image_name):
+                          image_name, validate_disk=True):
     """Returns validator for image could be used for current flavor
 
     :param flavor_name: defines which variable should be used
                        to get flavor id value.
     :param image_name: defines which variable should be used
                        to get image id value.
+    :param validate_disk: flag to indicate whether to validate flavor's disk.
+                          Should be True if instance is booted from image.
+                          Should be False if instance is booted from volume.
+                          Default value is True.
 
     """
     valid_result, flavor = _get_validated_flavor(config, clients, flavor_name)
@@ -360,7 +365,7 @@ def image_valid_on_flavor(config, clients, deployment, flavor_name,
                     "for requested image '%s'") % (flavor.id, image["id"])
         return ValidationResult(False, message)
 
-    if flavor.disk:
+    if flavor.disk and validate_disk:
         if image["size"] > flavor.disk * (1024 ** 3):
             message = _("The disk size for flavor '%s' is too small "
                         "for requested image '%s'") % (flavor.id, image["id"])
@@ -610,6 +615,37 @@ def required_openstack(config, clients, deployment, admin=False, users=False):
         return ValidationResult(False, _("Admin credentials required"))
 
 
+@validator
+def required_api_versions(config, clients, deployment, component, versions):
+    """Validator checks component API versions."""
+    versions = [str(v) for v in versions]
+    versions_str = ", ".join(versions)
+    msg = _("Task was designed to be used with %(component)s "
+            "V%(version)s, but V%(found_version)s is "
+            "selected.")
+    if component == "keystone":
+        if "2.0" not in versions and hasattr(clients.keystone(), "tenants"):
+            return ValidationResult(False, msg % {"component": component,
+                                                  "version": versions_str,
+                                                  "found_version": "2.0"})
+        if "3" not in versions and hasattr(clients.keystone(), "projects"):
+            return ValidationResult(False, msg % {"component": component,
+                                                  "version": versions_str,
+                                                  "found_version": "3"})
+    else:
+        used_version = config.get("context", {}).get("api_versions", {}).get(
+            component, {}).get("version",
+                               getattr(clients, component).choose_version())
+        if not used_version:
+            return ValidationResult(
+                False, _("Unable to determine the API version."))
+        if str(used_version) not in versions:
+            return ValidationResult(
+                False, msg % {"component": component,
+                              "version": versions_str,
+                              "found_version": used_version})
+
+
 @validator
 def volume_type_exists(config, clients, deployment, param_name):
     """Returns validator for volume types.
