@@ -15,6 +15,7 @@
 
 
 import os
+import re
 import shutil
 import subprocess
 import sys
@@ -22,7 +23,6 @@ import tempfile
 
 from oslo_utils import encodeutils
 
-from rally.common import costilius
 from rally.common.i18n import _
 from rally.common.io import subunit_v2
 from rally.common import logging
@@ -43,7 +43,7 @@ class TempestSetupFailure(exceptions.RallyException):
 def check_output(*args, **kwargs):
     kwargs["stderr"] = subprocess.STDOUT
     try:
-        output = costilius.sp_check_output(*args, **kwargs)
+        output = subprocess.check_output(*args, **kwargs)
     except subprocess.CalledProcessError as e:
         LOG.error("Failed cmd: '%s'" % e.cmd)
         LOG.error("Error output: '%s'" % encodeutils.safe_decode(e.output))
@@ -58,16 +58,21 @@ class Tempest(object):
     base_repo_dir = os.path.join(os.path.expanduser("~"),
                                  ".rally/tempest/base")
 
-    def __init__(self, deployment, verification=None,
-                 tempest_config=None, source=None, system_wide=False):
-        self.tempest_source = source or TEMPEST_SOURCE
+    def __init__(self, deployment, verification=None, tempest_config=None,
+                 source=None, version=None, plugin_source=None,
+                 plugin_version=None, system_wide=False):
         self.deployment = deployment
+        self.verification = verification
         self._path = os.path.join(os.path.expanduser("~"),
                                   ".rally/tempest",
                                   "for-deployment-%s" % deployment)
         self.config_file = tempest_config or self.path("tempest.conf")
+        self.tempest_source = source or TEMPEST_SOURCE
+        self.version = version
+        self.plugin_source = plugin_source
+        self.plugin_version = plugin_version
         self.log_file_raw = self.path("subunit.stream")
-        self.verification = verification
+
         self._env = None
         self._base_repo = None
         self._system_wide = system_wide
@@ -181,23 +186,8 @@ class Tempest(object):
             LOG.debug("No virtual environment for Tempest found.")
             LOG.info(_("Installing the virtual environment for Tempest."))
             LOG.debug("Virtual environment directory: %s" % path_to_venv)
-            required_vers = (2, 7)
-            if sys.version_info[:2] != required_vers:
-                # NOTE(andreykurilin): let's try to find a suitable python
-                # interpreter for Tempest
-                python_interpreter = costilius.get_interpreter(required_vers)
-                if not python_interpreter:
-                    raise exceptions.IncompatiblePythonVersion(
-                        version=sys.version, required_version=required_vers)
-                LOG.info(
-                    _("Tempest requires Python %(required)s, '%(found)s' was "
-                      "found in your system and it will be used for installing"
-                      " virtual environment.") % {"required": required_vers,
-                                                  "found": python_interpreter})
-            else:
-                python_interpreter = sys.executable
             try:
-                check_output(["virtualenv", "-p", python_interpreter, ".venv"],
+                check_output(["virtualenv", "-p", sys.executable, ".venv"],
                              cwd=self.path())
                 # NOTE(kun): Using develop mode installation is for run
                 #            multiple tempest instance. However, dependency
@@ -227,13 +217,17 @@ class Tempest(object):
         """
         if not self.is_configured() or override:
             if not override:
-                LOG.info(_("Tempest is not configured."))
+                LOG.info(_("Tempest is not configured "
+                           "for deployment: %s") % self.deployment)
 
-            LOG.info(_("Starting: Creating configuration file for Tempest."))
+            LOG.info(_("Creating Tempest configuration "
+                       "file for deployment: %s") % self.deployment)
             config.TempestConfig(self.deployment).generate(self.config_file)
-            LOG.info(_("Completed: Creating configuration file for Tempest."))
+            LOG.info(_("Tempest configuration file "
+                       "has been successfully created!"))
         else:
-            LOG.info("Tempest is already configured.")
+            LOG.info(_("Tempest is already configured "
+                       "for deployment: %s") % self.deployment)
 
     def _initialize_testr(self):
         if not os.path.isdir(self.path(".testrepository")):
@@ -269,12 +263,17 @@ class Tempest(object):
     def install(self):
         """Creates local Tempest repo and virtualenv for deployment."""
         if not self.is_installed():
+            LOG.info(_("Tempest is not installed "
+                       "for deployment: %s") % self.deployment)
+            LOG.info(_("Installing Tempest "
+                       "for deployment: %s") % self.deployment)
             try:
                 if not os.path.exists(self.path()):
                     if not self._is_git_repo(self.base_repo):
                         self._clone()
                     shutil.copytree(self.base_repo, self.path())
-                    for cmd in ["git", "checkout", "master"], ["git", "pull"]:
+                    if self.version:
+                        cmd = ["git", "checkout", self.version]
                         subprocess.check_call(cmd, cwd=self.path("tempest"))
                 if not self._system_wide:
                     self._install_venv()
@@ -296,6 +295,22 @@ class Tempest(object):
         if os.path.exists(self.path()):
             shutil.rmtree(self.path())
 
+    def install_plugin(self):
+        """Install Tempest plugin for local Tempest repo."""
+        LOG.info(_("Installing Tempest plugin from %s for "
+                   "deployment: %s") % (self.plugin_source, self.deployment))
+        egg = re.sub("\.git$", "",
+                     os.path.basename(self.plugin_source.strip("/")))
+        version = self.plugin_version or "master"
+        cmd = ["pip", "install", "--no-deps",
+               "--src", self.path("plugins"), "-e",
+               "git+{0}@{1}#egg={2}".format(self.plugin_source, version, egg)]
+        if not self._system_wide:
+            cmd.insert(0, self.path("tools/with_venv.sh"))
+            cmd.remove("--no-deps")
+        check_output(cmd, cwd=self.path())
+        LOG.info(_("Tempest plugin has been successfully installed!"))
+
     @logging.log_verification_wrapper(LOG.info, _("Run verification."))
     def _prepare_and_run(self, set_name, regex, tests_file, concur, failing):
         if not self.is_configured():
@@ -336,9 +351,14 @@ class Tempest(object):
         :param concur: How many processes to use to run Tempest tests.
                        The default value (0) auto-detects CPU count
         """
-        if tempest_conf and os.path.isfile(tempest_conf):
+        if tempest_conf:
             self.config_file = tempest_conf
-        LOG.info(_("Tempest config file: %s") % self.config_file)
+        if os.path.isfile(self.config_file):
+            LOG.info(_("Using Tempest config file: %s") % self.config_file)
+        else:
+            msg = _("Tempest config file '%s' not found!") % self.config_file
+            LOG.error(msg)
+            raise exceptions.NotFoundException(message=msg)
 
         concur_args = "--concurrency %d" % concur
         if concur != 1:
