@@ -63,6 +63,8 @@ re_raises = re.compile(
     r"\s:raise[^s] *.*$|\s:raises *:.*$|\s:raises *[^:]+$")
 re_db_import = re.compile(r"^from rally.common import db")
 re_objects_import = re.compile(r"^from rally.common import objects")
+re_old_type_class = re.compile(r"^\s*class \w+(\(\))?:")
+re_datetime_alias = re.compile(r"^(from|import) datetime(?!\s+as\s+dt$)")
 
 
 def skip_ignored_lines(func):
@@ -152,7 +154,8 @@ def check_import_of_logging(logical_line, physical_line, filename):
 
     excluded_files = ["./rally/common/logging.py",
                       "./tests/unit/test_logging.py",
-                      "./tests/ci/rally_verify.py"]
+                      "./tests/ci/rally_verify.py",
+                      "./tests/ci/sync_requirements.py"]
 
     forbidden_imports = ["from oslo_log",
                          "import oslo_log",
@@ -475,6 +478,29 @@ def check_raises(physical_line, filename):
                        "in docstrings.")
 
 
+@skip_ignored_lines
+def check_old_type_class(logical_line, physical_line, filename):
+    """Use new-style Python classes
+
+    N355
+    """
+
+    if re_old_type_class.search(logical_line):
+        yield (0, "N355 This class does not inherit from anything and thus "
+                  "will be an old-style class by default. Try to inherit from "
+                  "``object`` or another new-style class.")
+
+
+@skip_ignored_lines
+def check_datetime_alias(logical_line, physical_line, filename):
+    """Ensure using ``dt`` as alias for ``datetime``
+
+    N356
+    """
+    if re_datetime_alias.search(logical_line):
+        yield (0, "N356 Please use ``dt`` as alias for ``datetime``.")
+
+
 @skip_ignored_lines
 def check_db_imports_in_cli(logical_line, physical_line, filename):
     """Ensure that CLI modules do not use ``rally.common.db``
@@ -520,5 +546,7 @@ def factory(register):
     register(check_dict_formatting_in_string)
     register(check_using_unicode)
     register(check_raises)
+    register(check_datetime_alias)
     register(check_db_imports_in_cli)
     register(check_objects_imports_in_cli)
+    register(check_old_type_class)
