@@ -145,8 +145,10 @@ class VerifyCommandsTestCase(test.TestCase):
 
     @mock.patch("rally.api.Verification.import_results")
     def test_import_results(self, mock_verification_import_results):
-        deployment_id = "fake_uuid"
-        mock_verification_import_results.return_value = (None, None)
+        deployment_id = "fake_deployment_uuid"
+        fake_verification = {"uuid": "fake_verification_uuid"}
+        mock_verification_import_results.return_value = (None,
+                                                         fake_verification)
         self.verify.import_results(deployment=deployment_id, do_use=False)
         default_set_name = ""
         default_log_file = None
@@ -160,7 +162,9 @@ class VerifyCommandsTestCase(test.TestCase):
         deployment_id = "fake_uuid"
         set_name = "fake_set_name"
         log_file = "fake_log_file"
-        mock_verification_import_results.return_value = (None, None)
+        fake_verification = {"uuid": "fake_verification_uuid"}
+        mock_verification_import_results.return_value = (None,
+                                                         fake_verification)
         self.verify.import_results(deployment=deployment_id, set_name=set_name,
                                    log_file=log_file, do_use=False)
 
@@ -183,6 +187,7 @@ class VerifyCommandsTestCase(test.TestCase):
 
         mock_verification_list.assert_called_once_with()
         mock_print_list.assert_called_once_with(verifications, fields,
+                                                normalize_field_names=True,
                                                 sortby_index=fields.index(
                                                     "Created at"))
 
@@ -208,8 +213,9 @@ class VerifyCommandsTestCase(test.TestCase):
 
         self.verify.show(verification_id)
 
-        self.assertEqual([mock.call([verification], fields=total_fields),
-                         mock.call(values, fields, sortby_index=0)],
+        self.assertEqual([mock.call([verification], fields=total_fields,
+                                    normalize_field_names=True,),
+                          mock.call(values, fields, sortby_index=0)],
                          mock_print_list.call_args_list)
         mock_verification.get.assert_called_once_with(verification_id)
 
@@ -424,7 +430,7 @@ class VerifyCommandsTestCase(test.TestCase):
         deployment_uuid = "d26ebebc-3a5f-4d0d-9021-0c883bd560f5"
         self.verify.install(deployment_uuid)
         mock_verification_install_tempest.assert_called_once_with(
-            deployment_uuid, None, False)
+            deployment_uuid, None, None, False)
 
     @mock.patch("rally.api.Verification.install_tempest")
     def test_install_with_source_specified(
@@ -433,7 +439,16 @@ class VerifyCommandsTestCase(test.TestCase):
         source = "/tmp/tempest"
         self.verify.install(deployment_uuid, source)
         mock_verification_install_tempest.assert_called_once_with(
-            deployment_uuid, source, False)
+            deployment_uuid, source, None, False)
+
+    @mock.patch("rally.api.Verification.install_tempest")
+    def test_install_with_version_specified(
+            self, mock_verification_install_tempest):
+        deployment_uuid = "206118f8-fa8e-43a6-a3d5-a7d047c4091a"
+        version = "206118f8"
+        self.verify.install(deployment_uuid, version=version)
+        mock_verification_install_tempest.assert_called_once_with(
+            deployment_uuid, None, version, False)
 
     @mock.patch("rally.api.Verification.uninstall_tempest")
     def test_uninstall(self, mock_verification_uninstall_tempest):
@@ -449,15 +464,6 @@ class VerifyCommandsTestCase(test.TestCase):
         mock_verification_reinstall_tempest.assert_called_once_with(
             deployment_uuid, None, None, False)
 
-    @mock.patch("rally.api.Verification.reinstall_tempest")
-    def test_reinstall_with_config_specified(
-            self, mock_verification_reinstall_tempest):
-        deployment_uuid = "83514de2-a770-4e28-82dd-2826b725e733"
-        tempest_conf = "/tmp/tempest.conf"
-        self.verify.reinstall(deployment_uuid, tempest_config=tempest_conf)
-        mock_verification_reinstall_tempest.assert_called_once_with(
-            deployment_uuid, tempest_conf, None, False)
-
     @mock.patch("rally.api.Verification.reinstall_tempest")
     def test_reinstall_with_source_specified(
             self, mock_verification_reinstall_tempest):
@@ -465,17 +471,34 @@ class VerifyCommandsTestCase(test.TestCase):
         source = "/tmp/tempest"
         self.verify.reinstall(deployment_uuid, source=source)
         mock_verification_reinstall_tempest.assert_called_once_with(
-            deployment_uuid, None, source, False)
+            deployment_uuid, source, None, False)
 
     @mock.patch("rally.api.Verification.reinstall_tempest")
-    def test_reinstall_with_config_and_source_specified(
+    def test_reinstall_with_version_specified(
             self, mock_verification_reinstall_tempest):
-        deployment_uuid = "f71fb1e2-c442-4889-aaf8-69754828f5f0"
-        tempest_conf = "/tmp/tempest.conf"
-        source = "/tmp/tempest"
-        self.verify.reinstall(deployment_uuid, tempest_conf, source)
+        deployment_uuid = "c0202924-fba7-442e-93dd-90bcd7282188"
+        version = "c0202924"
+        self.verify.reinstall(deployment_uuid, version=version)
         mock_verification_reinstall_tempest.assert_called_once_with(
-            deployment_uuid, tempest_conf, source, False)
+            deployment_uuid, None, version, False)
+
+    @mock.patch("rally.api.Verification.install_tempest_plugin")
+    def test_install_plugin_from_url(
+            self, mock_verification_install_tempest_plugin):
+        deployment_uuid = "83514de2-a770-4e28-82dd-2826b725e733"
+        url = "https://github.com/fake/plugin"
+        self.verify.installplugin(deployment_uuid, url)
+        mock_verification_install_tempest_plugin.assert_called_once_with(
+            deployment_uuid, url, None, False)
+
+    @mock.patch("rally.api.Verification.install_tempest_plugin")
+    def test_install_plugin_from_path(
+            self, mock_verification_install_tempest_plugin):
+        deployment_uuid = "83514de2-a770-4e28-82dd-2826b725e733"
+        path = "/tmp/fake/plugin"
+        self.verify.installplugin(deployment_uuid, path)
+        mock_verification_install_tempest_plugin.assert_called_once_with(
+            deployment_uuid, path, None, False)
 
     @mock.patch("rally.api.Verification.discover_tests")
     def test_discover(self, mock_verification_discover_tests):
