@@ -17,9 +17,13 @@ from boto import exception as boto_exception
 import ddt
 import mock
 from neutronclient.common import exceptions as neutron_exceptions
+from novaclient import exceptions as nova_exc
+from watcherclient.common.apiclient import exceptions as watcher_exceptions
 
 from rally.common import utils
 from rally.plugins.openstack.cleanup import resources
+from rally.plugins.openstack.scenarios.keystone import utils as kutils
+from rally.plugins.openstack.scenarios.nova import utils as nutils
 from tests.unit import test
 
 BASE = "rally.plugins.openstack.cleanup.resources"
@@ -115,6 +119,37 @@ class NovaFloatingIPsTestCase(test.TestCase):
         self.assertIsNone(fips.name())
 
 
+class NovaFlavorsTestCase(test.TestCase):
+
+    @mock.patch("%s.base.ResourceManager._manager" % BASE)
+    @mock.patch("rally.common.utils.name_matches_object")
+    def test_list(self, mock_name_matches_object,
+                  mock_resource_manager__manager):
+        flavors = [mock.MagicMock(name="rally_foo1"),
+                   mock.MagicMock(name="rally_foo2"),
+                   mock.MagicMock(name="foo3")]
+        mock_name_matches_object.side_effect = [False, True, True]
+        mock_resource_manager__manager().list.return_value = flavors
+        self.assertEqual(flavors[1:], resources.NovaFlavors().list())
+        mock_name_matches_object.assert_has_calls(
+            [mock.call(r.name, nutils.NovaScenario) for r in flavors])
+
+    @mock.patch("%s.base.ResourceManager._manager" % BASE)
+    def test_is_deleted(self, mock_resource_manager__manager):
+        exc = nova_exc.NotFound(404)
+        mock_resource_manager__manager().get.side_effect = exc
+        flavor = resources.NovaFlavors()
+        flavor.raw_resource = mock.MagicMock()
+        self.assertEqual(True, flavor.is_deleted())
+
+    @mock.patch("%s.base.ResourceManager._manager" % BASE)
+    def test_is_deleted_fail(self, mock_resource_manager__manager):
+        mock_resource_manager__manager().get.side_effect = TypeError()
+        flavor = resources.NovaFlavors()
+        flavor.raw_resource = mock.MagicMock()
+        self.assertRaises(TypeError, flavor.is_deleted)
+
+
 class NovaSecurityGroupTestCase(test.TestCase):
 
     @mock.patch("%s.base.ResourceManager._manager" % BASE)
@@ -462,14 +497,18 @@ class GlanceImageTestCase(test.TestCase):
 
     def test_delete(self):
         glance = resources.GlanceImage()
+        glance._client = mock.Mock()
         glance._wrapper = mock.Mock()
         glance.raw_resource = mock.Mock()
 
-        self.assertEqual(
-            glance.delete(),
-            glance._wrapper.return_value.delete_image.return_value)
-        glance._wrapper.return_value.delete_image.assert_called_once_with(
-            glance.raw_resource)
+        client = glance._client.return_value
+        wrapper = glance._wrapper.return_value
+
+        deleted_image = mock.Mock(status="DELETED")
+        wrapper.get_image.side_effect = [glance.raw_resource, deleted_image]
+
+        glance.delete()
+        client().images.delete.assert_called_once_with(glance.raw_resource.id)
 
 
 class CeilometerTestCase(test.TestCase):
@@ -532,22 +571,25 @@ class KeystoneMixinTestCase(test.TestCase):
         mock_wrap().delete_some_resource.assert_called_once_with("id_a")
 
     @mock.patch(
-        "rally.plugins.openstack.scenarios.keystone.utils.is_temporary")
+        "rally.common.utils.name_matches_object")
     @mock.patch("%s.keystone_wrapper.wrap" % BASE)
-    def test_list(self, mock_wrap, mock_is_temporary):
+    def test_list(self, mock_wrap, mock_name_matches_object):
         keystone_mixin = self.get_keystone_mixin()
-        keystone_mixin._resource = "some_resource2"
+        keystone_mixin._resource = "fake_resource"
         keystone_mixin.admin = mock.MagicMock()
 
-        result = [mock.MagicMock(), mock.MagicMock(), mock.MagicMock()]
-        mock_is_temporary.side_effect = [True, True, False]
+        result = [mock.MagicMock(name="rally_foo1"),
+                  mock.MagicMock(name="rally_foo2"),
+                  mock.MagicMock(name="foo3")]
+        mock_name_matches_object.side_effect = [True, True, False]
 
-        mock_wrap().list_some_resource2s.return_value = result
+        mock_wrap().list_fake_resources.return_value = result
 
         self.assertSequenceEqual(result[:2], keystone_mixin.list())
-        mock_wrap().list_some_resource2s.assert_called_once_with()
+        mock_wrap().list_fake_resources.assert_called_once_with()
 
-        mock_is_temporary.assert_has_calls([mock.call(r) for r in result])
+        mock_name_matches_object.assert_has_calls(
+            [mock.call(r.name, kutils.KeystoneScenario) for r in result])
 
 
 class SwiftMixinTestCase(test.TestCase):
@@ -712,3 +754,29 @@ class FuelEnvironmentTestCase(test.TestCase):
 
         fres = resources.FuelEnvironment()
         self.assertEqual(envs[:-1], fres.list())
+
+
+class WatcherTemplateTestCase(test.TestCase):
+
+    def test_id(self):
+        watcher = resources.WatcherTemplate()
+        watcher.raw_resource = mock.MagicMock(uuid=100)
+        self.assertEqual(100, watcher.id())
+
+    @mock.patch("%s.WatcherTemplate._manager" % BASE)
+    def test_is_deleted(self, mock__manager):
+        mock__manager.return_value.get.return_value = None
+        watcher = resources.WatcherTemplate()
+        watcher.id = mock.Mock()
+        self.assertFalse(watcher.is_deleted())
+        mock__manager.side_effect = [watcher_exceptions.NotFound()]
+        self.assertTrue(watcher.is_deleted())
+
+    def test_list(self):
+        watcher = resources.WatcherTemplate()
+        watcher._manager = mock.MagicMock()
+
+        watcher.list()
+
+        self.assertEqual("audit_template", watcher._resource)
+        watcher._manager().list.assert_called_once_with(limit=0)
