@@ -26,16 +26,34 @@ CTX = "rally.plugins.openstack.context.keystone.users"
 
 class UserContextMixinTestCase(test.TestCase):
 
+    def setUp(self):
+        super(self.__class__, self).setUp()
+        self.mixin = users.UserContextMixin()
+        self.mixin.context = {
+            "config": {
+                "users": {
+                    "user_choice_method": "random",
+                },
+            },
+        }
+
     @mock.patch("%s.random.choice" % CTX, side_effect=lambda x: x[1])
-    def test_map_for_scenario(self, mock_choice):
+    def test_map_for_scenario_random(self, mock_choice):
+        self.mixin.context["config"]["users"]["user_choice_method"] = (
+            "random")
         users_ = []
         tenants = {}
 
-        for i in range(2):
-            tenants[str(i)] = {"name": str(i)}
-            for j in range(3):
-                users_.append({"id": "%s_%s" % (i, j),
-                              "tenant_id": str(i), "endpoint": "endpoint"})
+        for i in ("0", "1"):
+            tenants[i] = {"id": i, "name": i, "users": []}
+            for j in ("a", "b", "c"):
+                user = {
+                    "id": "%s_%s" % (i, j),
+                    "tenant_id": i,
+                    "credential": "credential",
+                }
+                users_.append(user)
+                tenants[i]["users"].append(user)
 
         context = {
             "admin": mock.MagicMock(),
@@ -44,21 +62,70 @@ class UserContextMixinTestCase(test.TestCase):
             "some_random_key": {
                 "nested": mock.MagicMock(),
                 "one_more": 10
-            }
+            },
+            "config": {
+                "users": {
+                    "user_choice_method": "random",
+                },
+            },
         }
         chosen_tenant = context["tenants"][context["users"][1]["tenant_id"]]
         expected_context = {
             "admin": context["admin"],
             "user": context["users"][1],
             "tenant": chosen_tenant,
-            "some_random_key": context["some_random_key"]
+            "some_random_key": context["some_random_key"],
+            "config": context["config"]
         }
 
         self.assertEqual(
             expected_context,
-            users.UserContextMixin().map_for_scenario(context)
+            self.mixin.map_for_scenario(context)
         )
 
+    def test_user_choice_method_default(self):
+        self.mixin.context["config"]["users"]["user_choice_method"] = None
+
+        self.assertEqual(
+            "random",
+            self.mixin.user_choice_method
+        )
+
+    @mock.patch("%s.random.choice" % CTX,
+                side_effect=Exception("Should not be raised"))
+    def test_map_for_scenario_round_robin(self, mock_choice):
+        self.mixin.context["config"]["users"]["user_choice_method"] = (
+            "round_robin")
+        tenants = {s: {"name": s, "users": []} for s in ("0", "1")}
+        users_ = []
+        for tenant_id, tenant in tenants.items():
+            for i in ("0", "1"):
+                user = {"id": "%s_%s" % (tenant_id, i), "tenant_id": tenant_id,
+                        "endpoint": "endpoint"}
+                users_.append(user)
+                tenant["users"].append(user)
+        context = {
+            "admin": mock.MagicMock(),
+            "users": users_,
+            "tenants": tenants,
+            "some_random_key": {
+                "nested": mock.MagicMock(),
+                "one_more": 10
+            },
+            "config": {
+                "users": {
+                    "user_choice_method": "round_robin",
+                },
+            },
+        }
+        expected_ids = ["0_0", "1_0", "0_1", "1_1"] * 4
+        mapped_ids = []
+        for i in range(16):
+            context["iteration"] = i
+            user = self.mixin.map_for_scenario(context)
+            mapped_ids.append(user["user"]["id"])
+        self.assertEqual(expected_ids, mapped_ids)
+
 
 class UserGeneratorTestCase(test.ScenarioTestCase):
 
@@ -79,7 +146,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
                     "resource_management_workers": self.threads,
                 }
             },
-            "admin": {"endpoint": mock.MagicMock()},
+            "admin": {"credential": mock.MagicMock()},
             "users": [],
             "task": {"uuid": "task_id"}
         })
@@ -117,7 +184,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
 
         user_generator._remove_default_security_group()
 
-        mock_wrap.assert_called_once_with(admin_clients)
+        mock_wrap.assert_called_once_with(admin_clients, user_generator)
         net_wrapper.supports_extension.assert_called_once_with(
             "security-group")
 
@@ -147,12 +214,13 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
 
         user_generator._remove_default_security_group()
 
-        mock_network.wrap.assert_called_once_with(admin_clients)
+        mock_network.wrap.assert_called_once_with(admin_clients,
+                                                  user_generator)
 
         mock_iterate_per_tenants.assert_called_once_with(
             user_generator.context["users"])
-        expected = [mock.call(user_generator.endpoint)] + [
-            mock.call(u["endpoint"])
+        expected = [mock.call(user_generator.credential)] + [
+            mock.call(u["credential"])
             for u, t in mock_iterate_per_tenants.return_value]
         self.osclients.Clients.assert_has_calls(expected, any_order=True)
 
@@ -220,28 +288,26 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
                                                           "nova-network")
         nova_admin.networks.disassociate.assert_called_once_with(networks[0])
 
-    @mock.patch("%s.broker.time.sleep" % CTX)
     @mock.patch("%s.keystone" % CTX)
-    def test__create_tenants(self, mock_keystone, mock_sleep):
+    def test__create_tenants(self, mock_keystone):
+        self.context["config"]["users"]["tenants"] = 1
         user_generator = users.UserGenerator(self.context)
-        user_generator.config["tenants"] = 1
         tenants = user_generator._create_tenants()
         self.assertEqual(1, len(tenants))
         id, tenant = tenants.popitem()
         self.assertIn("name", tenant)
 
-    @mock.patch("%s.broker.time.sleep" % CTX)
     @mock.patch("%s.keystone" % CTX)
-    def test__create_users(self, mock_keystone, mock_sleep):
+    def test__create_users(self, mock_keystone):
+        self.context["config"]["users"]["users_per_tenant"] = 2
         user_generator = users.UserGenerator(self.context)
         user_generator.context["tenants"] = {"t1": {"id": "t1", "name": "t1"},
                                              "t2": {"id": "t2", "name": "t2"}}
-        user_generator.config["users_per_tenant"] = 2
         users_ = user_generator._create_users()
         self.assertEqual(4, len(users_))
         for user in users_:
             self.assertIn("id", user)
-            self.assertIn("endpoint", user)
+            self.assertIn("credential", user)
 
     @mock.patch("%s.keystone" % CTX)
     def test__delete_tenants(self, mock_keystone):
@@ -298,61 +364,63 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
         self.assertEqual(len(ctx.context["users"]), 0)
         self.assertEqual(len(ctx.context["tenants"]), 0)
 
+    @mock.patch("rally.common.broker.LOG.warning")
     @mock.patch("%s.keystone" % CTX)
-    def test_setup_and_cleanup_failure(self, mock_keystone):
+    def test_setup_and_cleanup_with_error_during_create_user(
+            self, mock_keystone, mock_log_warning):
         wrapped_keystone = mock_keystone.wrap.return_value
-        wrapped_keystone.create_user.side_effect = Exception()
+        wrapped_keystone.create_user.side_effect = Exception
         with users.UserGenerator(self.context) as ctx:
-            self.assertRaises(exceptions.ContextSetupFailure, ctx.setup)
+                self.assertRaises(exceptions.ContextSetupFailure, ctx.setup)
+                mock_log_warning.assert_called_with(
+                    "Failed to consume a task from the queue: ")
 
         # Ensure that tenants get deleted anyway
-        self.assertEqual(len(ctx.context["tenants"]), 0)
+        self.assertEqual(0, len(ctx.context["tenants"]))
 
     @mock.patch("%s.keystone" % CTX)
     def test_users_and_tenants_in_context(self, mock_keystone):
         wrapped_keystone = mock.MagicMock()
         mock_keystone.wrap.return_value = wrapped_keystone
 
-        endpoint = objects.Endpoint("foo_url", "foo", "foo_pass",
-                                    https_insecure=True,
-                                    https_cacert="cacert")
+        credential = objects.Credential("foo_url", "foo", "foo_pass",
+                                        https_insecure=True,
+                                        https_cacert="cacert")
         tmp_context = dict(self.context)
         tmp_context["config"]["users"] = {"tenants": 1,
                                           "users_per_tenant": 2,
                                           "resource_management_workers": 1}
-        tmp_context["admin"]["endpoint"] = endpoint
+        tmp_context["admin"]["credential"] = credential
 
-        endpoint_dict = endpoint.to_dict(False)
+        credential_dict = credential.to_dict(False)
         user_list = [mock.MagicMock(id="id_%d" % i)
                      for i in range(self.users_num)]
         wrapped_keystone.create_user.side_effect = user_list
 
         with users.UserGenerator(tmp_context) as ctx:
+            ctx.generate_random_name = mock.Mock()
             ctx.setup()
 
             create_tenant_calls = []
             for i, t in enumerate(ctx.context["tenants"]):
-                pattern = users.UserGenerator.PATTERN_TENANT
                 create_tenant_calls.append(
-                    mock.call(
-                        pattern % {"task_id": tmp_context["task"]["uuid"],
-                                   "iter": i},
-                        ctx.config["project_domain"]))
+                    mock.call(ctx.generate_random_name.return_value,
+                              ctx.config["project_domain"]))
 
             for user in ctx.context["users"]:
-                self.assertEqual(set(["id", "endpoint", "tenant_id"]),
+                self.assertEqual(set(["id", "credential", "tenant_id"]),
                                  set(user.keys()))
 
-                user_endpoint_dict = user["endpoint"].to_dict(False)
+                user_credential_dict = user["credential"].to_dict(False)
 
                 excluded_keys = ["auth_url", "username", "password",
                                  "tenant_name", "region_name",
                                  "project_domain_name",
                                  "admin_domain_name",
                                  "user_domain_name"]
-                for key in (set(endpoint_dict.keys()) - set(excluded_keys)):
-                    self.assertEqual(endpoint_dict[key],
-                                     user_endpoint_dict[key])
+                for key in (set(credential_dict.keys()) - set(excluded_keys)):
+                    self.assertEqual(credential_dict[key],
+                                     user_credential_dict[key])
 
             tenants_ids = []
             for t in ctx.context["tenants"].keys():
@@ -365,8 +433,9 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
 
     @mock.patch("%s.keystone" % CTX)
     def test_users_contains_correct_endpoint_type(self, mock_keystone):
-        endpoint = objects.Endpoint("foo_url", "foo", "foo_pass",
-                                    endpoint_type=consts.EndpointType.INTERNAL)
+        credential = objects.Credential(
+            "foo_url", "foo", "foo_pass",
+            endpoint_type=consts.EndpointType.INTERNAL)
         config = {
             "config": {
                 "users": {
@@ -375,7 +444,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
                     "resource_management_workers": 1
                 }
             },
-            "admin": {"endpoint": endpoint},
+            "admin": {"credential": credential},
             "task": {"uuid": "task_id"}
         }
 
@@ -383,11 +452,11 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
         users_ = user_generator._create_users()
 
         for user in users_:
-            self.assertEqual("internal", user["endpoint"].endpoint_type)
+            self.assertEqual("internal", user["credential"].endpoint_type)
 
     @mock.patch("%s.keystone" % CTX)
     def test_users_contains_default_endpoint_type(self, mock_keystone):
-        endpoint = objects.Endpoint("foo_url", "foo", "foo_pass")
+        credential = objects.Credential("foo_url", "foo", "foo_pass")
         config = {
             "config": {
                 "users": {
@@ -396,7 +465,7 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
                     "resource_management_workers": 1
                 }
             },
-            "admin": {"endpoint": endpoint},
+            "admin": {"credential": credential},
             "task": {"uuid": "task_id"}
         }
 
@@ -404,4 +473,4 @@ class UserGeneratorTestCase(test.ScenarioTestCase):
         users_ = user_generator._create_users()
 
         for user in users_:
-            self.assertEqual("public", user["endpoint"].endpoint_type)
+            self.assertEqual("public", user["credential"].endpoint_type)
