@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ddt
 import mock
 
 from rally.plugins.openstack.scenarios.cinder import volumes
@@ -26,6 +27,7 @@ class fake_type(object):
     name = "fake"
 
 
+@ddt.ddt
 class CinderServersTestCase(test.ScenarioTestCase):
 
     def _get_context(self):
@@ -34,7 +36,7 @@ class CinderServersTestCase(test.ScenarioTestCase):
             "user": {"tenant_id": "fake",
                      "credential": mock.MagicMock()},
             "tenant": {"id": "fake", "name": "fake",
-                       "volumes": [{"id": "uuid"}],
+                       "volumes": [{"id": "uuid", "size": 1}],
                        "servers": [1]}})
         return context
 
@@ -196,7 +198,12 @@ class CinderServersTestCase(test.ScenarioTestCase):
         scenario._create_volume = mock.MagicMock(return_value=fake_volume)
         scenario._delete_volume = mock.MagicMock()
 
-        scenario.create_and_attach_volume(10, "img", "0")
+        volume_args = {"some_key": "some_val"}
+        vm_args = {"some_key": "some_val"}
+
+        scenario.create_and_attach_volume(10, "img", "0",
+                                          create_volume_params=volume_args,
+                                          create_vm_params=vm_args)
         scenario._attach_volume.assert_called_once_with(fake_server,
                                                         fake_volume)
         scenario._detach_volume.assert_called_once_with(fake_server,
@@ -216,12 +223,14 @@ class CinderServersTestCase(test.ScenarioTestCase):
         scenario._delete_volume = mock.MagicMock()
         scenario._delete_image = mock.MagicMock()
 
-        scenario.create_and_upload_volume_to_image(2,
+        scenario.create_and_upload_volume_to_image(2, image="img",
                                                    container_format="fake",
                                                    disk_format="disk",
-                                                   do_delete=False)
+                                                   do_delete=False,
+                                                   fakeargs="fakeargs")
 
-        scenario._create_volume.assert_called_once_with(2)
+        scenario._create_volume.assert_called_once_with(2, imageRef="img",
+                                                        fakeargs="fakeargs")
         scenario._upload_volume_to_image.assert_called_once_with(fake_volume,
                                                                  False,
                                                                  "fake",
@@ -229,9 +238,11 @@ class CinderServersTestCase(test.ScenarioTestCase):
         scenario._create_volume.reset_mock()
         scenario._upload_volume_to_image.reset_mock()
 
-        scenario.create_and_upload_volume_to_image(1, do_delete=True)
+        scenario.create_and_upload_volume_to_image(1, image=None,
+                                                   do_delete=True,
+                                                   fakeargs="fakeargs")
 
-        scenario._create_volume.assert_called_once_with(1)
+        scenario._create_volume.assert_called_once_with(1, fakeargs="fakeargs")
         scenario._upload_volume_to_image.assert_called_once_with(fake_volume,
                                                                  False,
                                                                  "bare",
@@ -331,6 +342,84 @@ class CinderServersTestCase(test.ScenarioTestCase):
         self.assertEqual(scenario._delete_snapshot.call_count, snapshots_count)
         self.assertEqual(scenario._detach_volume.call_count, attached_count)
 
+    def test_create_nested_snapshots_and_attach_volume_kwargs(self):
+        fake_volume = mock.MagicMock()
+        fake_snapshot = mock.MagicMock()
+
+        scenario = volumes.CinderVolumes(context=self._get_context())
+
+        scenario._attach_volume = mock.MagicMock()
+        scenario._detach_volume = mock.MagicMock()
+        scenario._delete_server = mock.MagicMock()
+        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
+        scenario._delete_volume = mock.MagicMock()
+        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
+        scenario._delete_snapshot = mock.MagicMock()
+
+        volume_kwargs = {"volume_type": "type1"}
+        scenario.create_nested_snapshots_and_attach_volume(
+            size={"min": 1, "max": 1},
+            create_volume_kwargs=volume_kwargs)
+
+        scenario._create_volume.assert_called_once_with(1, **volume_kwargs)
+        self.assertEqual(fake_volume, scenario._create_volume.return_value)
+
+    def test_create_nested_snapshots_and_attach_volume_snapshot_kwargs(self):
+        fake_volume = mock.MagicMock()
+        fake_volume.id = "FAKE_ID"
+        fake_snapshot = mock.MagicMock()
+
+        scenario = volumes.CinderVolumes(context=self._get_context())
+
+        scenario._attach_volume = mock.MagicMock()
+        scenario._detach_volume = mock.MagicMock()
+        scenario._delete_server = mock.MagicMock()
+        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
+        scenario._delete_volume = mock.MagicMock()
+        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
+        scenario._delete_snapshot = mock.MagicMock()
+
+        volume_kwargs = {"volume_type": "type1"}
+        snapshot_kwargs = {"name": "snapshot1", "description": "snaphot one"}
+        scenario.create_nested_snapshots_and_attach_volume(
+            size={"min": 1, "max": 1},
+            create_volume_kwargs=volume_kwargs,
+            create_snapshot_kwargs=snapshot_kwargs
+        )
+
+        scenario._create_snapshot.assert_called_once_with(fake_volume.id,
+                                                          False,
+                                                          **snapshot_kwargs)
+        self.assertEqual(fake_snapshot, scenario._create_snapshot.return_value)
+
+    def test_create_nested_snapshots_and_attach_volume_deprecate_kwargs(self):
+        fake_volume = mock.MagicMock()
+        fake_volume.id = "FAKE_ID"
+        fake_snapshot = mock.MagicMock()
+
+        scenario = volumes.CinderVolumes(context=self._get_context())
+
+        scenario._attach_volume = mock.MagicMock()
+        scenario._detach_volume = mock.MagicMock()
+        scenario._delete_server = mock.MagicMock()
+        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
+        scenario._delete_volume = mock.MagicMock()
+        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
+        scenario._delete_snapshot = mock.MagicMock()
+
+        volume_kwargs = {"volume_type": "type1"}
+        snapshot_kwargs = {"name": "snapshot1", "description": "snaphot one"}
+        scenario.create_nested_snapshots_and_attach_volume(
+            size={"min": 1, "max": 1},
+            create_volume_kwargs=volume_kwargs,
+            **snapshot_kwargs
+        )
+
+        scenario._create_snapshot.assert_called_once_with(fake_volume.id,
+                                                          False,
+                                                          **snapshot_kwargs)
+        self.assertEqual(fake_snapshot, scenario._create_snapshot.return_value)
+
     def test_create_nested_snapshots_calls_order(self):
         fake_volume1 = mock.MagicMock()
         fake_volume2 = mock.MagicMock()
@@ -485,3 +574,66 @@ class CinderServersTestCase(test.ScenarioTestCase):
         scenario._list_backups.assert_called_once_with(True)
         self.assertFalse(scenario._delete_volume.called)
         self.assertFalse(scenario._delete_backup.called)
+
+    @ddt.data({},
+              {"nested_level": 2},
+              {"image": "img"})
+    @ddt.unpack
+    def test_create_volume_and_clone(self, nested_level=1,
+                                     image=None):
+        create_volumes_count = nested_level + 1
+        fake_volumes = [mock.Mock(size=1) for i in range(create_volumes_count)]
+        scenario = volumes.CinderVolumes(self.context)
+        scenario._create_volume = mock.MagicMock(side_effect=fake_volumes)
+
+        scenario.create_volume_and_clone(1, image=image,
+                                         nested_level=nested_level,
+                                         fakearg="fake")
+
+        expected = [mock.call(1, imageRef=image, fakearg="fake")
+                    if image else mock.call(1, fakearg="fake")]
+        for i in range(nested_level):
+            expected.append(mock.call(fake_volumes[i].size,
+                                      source_volid=fake_volumes[i].id,
+                                      atomic_action=False, fakearg="fake")
+                            )
+            self._test_atomic_action_timer(scenario.atomic_actions(),
+                                           "cinder.clone_volume")
+        scenario._create_volume.assert_has_calls(expected)
+
+    def test_create_volume_from_snapshot(self):
+        fake_snapshot = mock.MagicMock(id=1)
+        fake_volume = mock.MagicMock()
+        create_snapshot_args = {"force": False}
+        scenario = volumes.CinderVolumes(self._get_context())
+
+        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
+        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
+        scenario._delete_snapshot = mock.MagicMock()
+        scenario._delete_volume = mock.MagicMock()
+
+        scenario.create_volume_from_snapshot(fakearg="f")
+
+        scenario._create_snapshot.assert_called_once_with("uuid")
+        scenario._create_volume.assert_called_once_with(
+            1, snapshot_id=fake_snapshot.id, fakearg="f")
+        scenario._delete_snapshot.assert_called_once_with(fake_snapshot)
+        scenario._delete_volume.assert_called_once_with(fake_volume)
+
+        scenario._create_snapshot.reset_mock()
+        scenario._create_volume.reset_mock()
+        scenario._delete_snapshot.reset_mock()
+        scenario._delete_volume.reset_mock()
+
+        scenario.create_volume_from_snapshot(
+            do_delete=False,
+            create_snapshot_kwargs=create_snapshot_args,
+            fakearg="f"
+        )
+
+        scenario._create_snapshot.assert_called_once_with(
+            "uuid", **create_snapshot_args)
+        scenario._create_volume.assert_called_once_with(
+            1, snapshot_id=fake_snapshot.id, fakearg="f")
+        self.assertFalse(scenario._delete_snapshot.called)
+        self.assertFalse(scenario._delete_volume.called)
