@@ -320,11 +320,30 @@ class NovaServersTestCase(test.ScenarioTestCase):
         scenario.list_servers(True)
         scenario._list_servers.assert_called_once_with(True)
 
+    def test_boot_server_from_volume(self):
+        fake_server = object()
+        scenario = servers.NovaServers(self.context)
+        scenario._boot_server = mock.MagicMock(return_value=fake_server)
+
+        fake_volume = fakes.FakeVolumeManager().create()
+        fake_volume.id = "volume_id"
+        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
+
+        scenario.boot_server_from_volume("img", 0, 5, volume_type=None,
+                                         auto_assign_nic=False,
+                                         fakearg="f")
+
+        scenario._create_volume.assert_called_once_with(5, imageRef="img",
+                                                        volume_type=None)
+        scenario._boot_server.assert_called_once_with(
+            None, 0, auto_assign_nic=False,
+            block_device_mapping={"vda": "volume_id:::1"},
+            fakearg="f")
+
     def test_boot_server_from_volume_and_delete(self):
         fake_server = object()
         scenario = servers.NovaServers(self.context)
         scenario._boot_server = mock.MagicMock(return_value=fake_server)
-        scenario.generate_random_name = mock.MagicMock(return_value="name")
         scenario.sleep_between = mock.MagicMock()
         scenario._delete_server = mock.MagicMock()
 
@@ -332,12 +351,13 @@ class NovaServersTestCase(test.ScenarioTestCase):
         fake_volume.id = "volume_id"
         scenario._create_volume = mock.MagicMock(return_value=fake_volume)
 
-        scenario.boot_server_from_volume_and_delete("img", 0, 5, 10, 20,
+        scenario.boot_server_from_volume_and_delete("img", 0, 5, None, 10, 20,
                                                     fakearg="f")
 
-        scenario._create_volume.assert_called_once_with(5, imageRef="img")
+        scenario._create_volume.assert_called_once_with(5, imageRef="img",
+                                                        volume_type=None)
         scenario._boot_server.assert_called_once_with(
-            "img", 0,
+            None, 0,
             block_device_mapping={"vda": "volume_id:::1"},
             fakearg="f")
         scenario.sleep_between.assert_called_once_with(10, 20)
@@ -509,7 +529,7 @@ class NovaServersTestCase(test.ScenarioTestCase):
 
         scenario._create_volume.assert_called_once_with(10, imageRef="img")
         scenario._boot_server.assert_called_once_with(
-            "img", flavor,
+            None, flavor,
             block_device_mapping={"vda": "volume_id:::1"})
         scenario.sleep_between.assert_called_once_with(10, 20)
         scenario._resize.assert_called_once_with(fake_server, to_flavor)
@@ -567,14 +587,16 @@ class NovaServersTestCase(test.ScenarioTestCase):
         scenario._create_volume = mock.MagicMock(return_value=fake_volume)
 
         scenario.boot_server_from_volume_and_live_migrate("img", 0, 5,
+                                                          volume_type=None,
                                                           min_sleep=10,
                                                           max_sleep=20,
                                                           fakearg="f")
 
-        scenario._create_volume.assert_called_once_with(5, imageRef="img")
+        scenario._create_volume.assert_called_once_with(5, imageRef="img",
+                                                        volume_type=None)
 
         scenario._boot_server.assert_called_once_with(
-            "img", 0,
+            None, 0,
             block_device_mapping={"vda": "volume_id:::1"},
             fakearg="f")
 
@@ -636,7 +658,6 @@ class NovaServersTestCase(test.ScenarioTestCase):
         scenario = servers.NovaServers(self.context)
         scenario.generate_random_name = mock.MagicMock(return_value="name")
         scenario._boot_server = mock.MagicMock(return_value=fake_server)
-        scenario._stop_server = mock.MagicMock()
         scenario._migrate = mock.MagicMock()
         scenario._resize_confirm = mock.MagicMock()
         scenario._resize_revert = mock.MagicMock()
@@ -650,16 +671,14 @@ class NovaServersTestCase(test.ScenarioTestCase):
                                                       fakearg="fakearg",
                                                       confirm=confirm)
 
-        scenario._stop_server.assert_called_once_with(fake_server)
-
         scenario._migrate.assert_called_once_with(fake_server)
 
         if confirm:
             scenario._resize_confirm.assert_called_once_with(fake_server,
-                                                             status="SHUTOFF")
+                                                             status="ACTIVE")
         else:
             scenario._resize_revert.assert_called_once_with(fake_server,
-                                                            status="SHUTOFF")
+                                                            status="ACTIVE")
 
         scenario._delete_server.assert_called_once_with(fake_server)
 
@@ -742,3 +761,38 @@ class NovaServersTestCase(test.ScenarioTestCase):
             tenant_id=server.tenant_id)
         scenario._associate_floating_ip.assert_called_once_with(
             server, net_wrap.create_floating_ip.return_value["ip"])
+
+    def test_boot_and_update_server(self):
+        scenario = servers.NovaServers(self.context)
+        scenario._boot_server = mock.Mock()
+        scenario._update_server = mock.Mock()
+
+        scenario.boot_and_update_server("img", "flavor",
+                                        "desp",
+                                        fakearg="fakearg")
+        scenario._boot_server.assert_called_once_with("img", "flavor",
+                                                      fakearg="fakearg")
+        scenario._update_server.assert_called_once_with(
+            scenario._boot_server.return_value, "desp")
+
+    def test_boot_server_from_volume_snapshot(self):
+        fake_volume = mock.MagicMock(id="volume_id")
+        fake_snapshot = mock.MagicMock(id="snapshot_id")
+
+        scenario = servers.NovaServers(self.context)
+        scenario._boot_server = mock.MagicMock()
+        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
+        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
+
+        scenario.boot_server_from_volume_snapshot("img", "flavor", 1,
+                                                  volume_type=None,
+                                                  auto_assign_nic=False,
+                                                  fakearg="f")
+
+        scenario._create_volume.assert_called_once_with(1, imageRef="img",
+                                                        volume_type=None)
+        scenario._create_snapshot.assert_called_once_with("volume_id", False)
+        scenario._boot_server.assert_called_once_with(
+            None, "flavor", auto_assign_nic=False,
+            block_device_mapping={"vda": "snapshot_id:snap::1"},
+            fakearg="f")
