@@ -37,7 +37,6 @@ class NovaScenarioTestCase(test.ScenarioTestCase):
         self.volume = mock.Mock()
         self.floating_ip = mock.Mock()
         self.image = mock.Mock()
-        self.keypair = mock.Mock()
         self.context["iteration"] = 3
         self.context["config"] = {"users": {"tenants": 2}}
 
@@ -407,19 +406,21 @@ class NovaScenarioTestCase(test.ScenarioTestCase):
     def test__force_delete_servers(self):
         self._test_delete_servers(force=True)
 
-    def test__delete_image(self):
+    @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
+    def test__delete_image(self, mock_wrap):
         nova_scenario = utils.NovaScenario(context=self.context)
+        nova_scenario._clients = mock.Mock()
         nova_scenario._delete_image(self.image)
-        self.image.delete.assert_called_once_with()
+        self.clients("glance").images.delete.assert_called_once_with(
+            self.image.id)
         self.mock_wait_for_status.mock.assert_called_once_with(
             self.image,
-            ready_statuses=["deleted"],
+            ready_statuses=["deleted", "pending_delete"],
             check_deletion=True,
-            update_resource=self.mock_get_from_manager.mock.return_value,
+            update_resource=mock_wrap.return_value.get_image,
             check_interval=CONF.benchmark.
             nova_server_image_delete_poll_interval,
             timeout=CONF.benchmark.nova_server_image_delete_timeout)
-        self.mock_get_from_manager.mock.assert_called_once_with()
         self._test_atomic_action_timer(nova_scenario.atomic_actions(),
                                        "nova.delete_image")
 
@@ -631,6 +632,8 @@ class NovaScenarioTestCase(test.ScenarioTestCase):
     def test__find_host_to_migrate(self):
         fake_server = self.server
         fake_host = {"nova-compute": {"available": True}}
+        fake_host_compute_off = {"nova-compute": {"available": False}}
+        fake_host_no_compute = {"nova-conductor": {"available": True}}
         self.admin_clients("nova").servers.get.return_value = fake_server
         self.admin_clients("nova").availability_zones.list.return_value = [
             mock.MagicMock(zoneName="a",
@@ -638,7 +641,8 @@ class NovaScenarioTestCase(test.ScenarioTestCase):
                                   "a3": fake_host}),
             mock.MagicMock(zoneName="b",
                            hosts={"b1": fake_host, "b2": fake_host,
-                                  "b3": fake_host}),
+                                  "b3": fake_host, "b4": fake_host_compute_off,
+                                  "b5": fake_host_no_compute}),
             mock.MagicMock(zoneName="c",
                            hosts={"c1": fake_host,
                                   "c2": fake_host, "c3": fake_host})
@@ -745,27 +749,32 @@ class NovaScenarioTestCase(test.ScenarioTestCase):
                                        "nova.list_security_groups")
 
     def test__list_keypairs(self):
-        keypairs_list = ["foo_keypair"]
-        self.clients("nova").keypairs.list.return_value = keypairs_list
-        nova_scenario = utils.NovaScenario(context=self.context)
-        return_keypairs_list = nova_scenario._list_keypairs()
-        self.assertEqual(keypairs_list, return_keypairs_list)
+        nova_scenario = utils.NovaScenario()
+        result = nova_scenario._list_keypairs()
+        self.assertEqual(self.clients("nova").keypairs.list.return_value,
+                         result)
+        self.clients("nova").keypairs.list.assert_called_once_with()
         self._test_atomic_action_timer(nova_scenario.atomic_actions(),
                                        "nova.list_keypairs")
 
     def test__create_keypair(self):
-        self.clients("nova").keypairs.create.return_value.name = self.keypair
-        nova_scenario = utils.NovaScenario(context=self.context)
-        return_keypair = nova_scenario._create_keypair()
-        self.assertEqual(self.keypair, return_keypair)
+        nova_scenario = utils.NovaScenario()
+        nova_scenario.generate_random_name = mock.Mock(
+            return_value="rally_nova_keypair_fake")
+        result = nova_scenario._create_keypair(fakeargs="fakeargs")
+        self.assertEqual(
+            self.clients("nova").keypairs.create.return_value.name,
+            result)
+        self.clients("nova").keypairs.create.assert_called_once_with(
+            "rally_nova_keypair_fake", fakeargs="fakeargs")
         self._test_atomic_action_timer(nova_scenario.atomic_actions(),
                                        "nova.create_keypair")
 
     def test__delete_keypair(self):
-        nova_scenario = utils.NovaScenario(context=self.context)
-        nova_scenario._delete_keypair(self.keypair)
+        nova_scenario = utils.NovaScenario()
+        nova_scenario._delete_keypair("fake_keypair")
         self.clients("nova").keypairs.delete.assert_called_once_with(
-            self.keypair)
+            "fake_keypair")
         self._test_atomic_action_timer(nova_scenario.atomic_actions(),
                                        "nova.delete_keypair")
 
@@ -805,8 +814,10 @@ class NovaScenarioTestCase(test.ScenarioTestCase):
                                        "nova.delete_floating_ips_bulk")
 
     def test__list_hypervisors(self):
-        nova_scenario = utils.NovaScenario(context=self.context)
-        nova_scenario._list_hypervisors(detailed=False)
+        nova_scenario = utils.NovaScenario()
+        result = nova_scenario._list_hypervisors(detailed=False)
+        self.assertEqual(
+            self.admin_clients("nova").hypervisors.list.return_value, result)
         self.admin_clients("nova").hypervisors.list.assert_called_once_with(
             False)
         self._test_atomic_action_timer(nova_scenario.atomic_actions(),
@@ -814,7 +825,9 @@ class NovaScenarioTestCase(test.ScenarioTestCase):
 
     def test__list_images(self):
         nova_scenario = utils.NovaScenario()
-        nova_scenario._list_images(detailed=False, fakearg="fakearg")
+        result = nova_scenario._list_images(detailed=False, fakearg="fakearg")
+        self.assertEqual(self.clients("nova").images.list.return_value,
+                         result)
         self.clients("nova").images.list.assert_called_once_with(
             False, fakearg="fakearg")
         self._test_atomic_action_timer(nova_scenario.atomic_actions(),
@@ -837,41 +850,154 @@ class NovaScenarioTestCase(test.ScenarioTestCase):
                                        "nova.unlock_server")
 
     def test__delete_network(self):
-        fake_netlabel = "test1"
         nova_scenario = utils.NovaScenario()
-        nova_scenario._delete_network(fake_netlabel)
+        result = nova_scenario._delete_network("fake_net_id")
+        self.assertEqual(
+            self.admin_clients("nova").networks.delete.return_value,
+            result)
         self.admin_clients("nova").networks.delete.assert_called_once_with(
-            fake_netlabel)
+            "fake_net_id")
         self._test_atomic_action_timer(nova_scenario.atomic_actions(),
                                        "nova.delete_network")
 
     @mock.patch(NOVA_UTILS + ".network_wrapper.generate_cidr")
     def test__create_network(self, mock_generate_cidr):
-        fake_cidr = "10.2.0.0/24"
-        fake_net = mock.MagicMock()
-        fake_net.cidr = fake_cidr
-        self.admin_clients("nova").networks.create.return_value = (fake_net)
-
         nova_scenario = utils.NovaScenario()
         nova_scenario.generate_random_name = mock.Mock(
             return_value="rally_novanet_fake")
 
-        return_netlabel = nova_scenario._create_network(fake_cidr,
-                                                        fakearg="fakearg")
-        mock_generate_cidr.assert_called_once_with(start_cidr=fake_cidr)
+        result = nova_scenario._create_network("fake_start_cidr",
+                                               fakearg="fakearg")
+
+        mock_generate_cidr.assert_called_once_with(
+            start_cidr="fake_start_cidr")
+
+        self.assertEqual(
+            self.admin_clients("nova").networks.create.return_value,
+            result)
         self.admin_clients("nova").networks.create.assert_called_once_with(
             label="rally_novanet_fake", cidr=mock_generate_cidr.return_value,
             fakearg="fakearg")
-        self.assertEqual(return_netlabel, fake_net)
         self._test_atomic_action_timer(nova_scenario.atomic_actions(),
                                        "nova.create_network")
 
     def test__list_flavors(self):
         nova_scenario = utils.NovaScenario()
-        self.clients("nova").flavors.list.return_value = "flavors_list"
         result = nova_scenario._list_flavors(detailed=True, fakearg="fakearg")
-        self.assertEqual("flavors_list", result)
+        self.assertEqual(self.clients("nova").flavors.list.return_value,
+                         result)
         self.clients("nova").flavors.list.assert_called_once_with(
             True, fakearg="fakearg")
         self._test_atomic_action_timer(nova_scenario.atomic_actions(),
                                        "nova.list_flavors")
+
+    @ddt.data({},
+              {"hypervisor": "foo_hypervisor"})
+    @ddt.unpack
+    def test__list_agents(self, hypervisor=None):
+        nova_scenario = utils.NovaScenario()
+        result = nova_scenario._list_agents(hypervisor)
+        self.assertEqual(
+            self.admin_clients("nova").agents.list.return_value, result)
+        self.admin_clients("nova").agents.list.assert_called_once_with(
+            hypervisor)
+        self._test_atomic_action_timer(nova_scenario.atomic_actions(),
+                                       "nova.list_agents")
+
+    def test__list_aggregates(self):
+        nova_scenario = utils.NovaScenario()
+        result = nova_scenario._list_aggregates()
+        self.assertEqual(
+            self.admin_clients("nova").aggregates.list.return_value, result)
+        self.admin_clients("nova").aggregates.list.assert_called_once_with()
+        self._test_atomic_action_timer(nova_scenario.atomic_actions(),
+                                       "nova.list_aggregates")
+
+    def test__list_availability_zones(self):
+        nova_scenario = utils.NovaScenario()
+        result = nova_scenario._list_availability_zones(detailed=True)
+        self.assertEqual(
+            self.admin_clients("nova").availability_zones.list.return_value,
+            result)
+        avail_zones_client = self.admin_clients("nova").availability_zones
+        avail_zones_client.list.assert_called_once_with(True)
+        self._test_atomic_action_timer(nova_scenario.atomic_actions(),
+                                       "nova.list_availbility_zones")
+
+    @ddt.data({},
+              {"zone": "foo_zone"})
+    @ddt.unpack
+    def test__list_hosts(self, zone=None):
+        nova_scenario = utils.NovaScenario()
+        result = nova_scenario._list_hosts(zone)
+        self.assertEqual(self.admin_clients("nova").hosts.list.return_value,
+                         result)
+        self.admin_clients("nova").hosts.list.assert_called_once_with(zone)
+        self._test_atomic_action_timer(nova_scenario.atomic_actions(),
+                                       "nova.list_hosts")
+
+    @ddt.data({},
+              {"host": "foo_host"},
+              {"binary": "foo_binary"},
+              {"host": "foo_host", "binary": "foo_binary"})
+    @ddt.unpack
+    def test__list_services(self, host=None, binary=None):
+        nova_scenario = utils.NovaScenario()
+        result = nova_scenario._list_services(host=host, binary=binary)
+        self.assertEqual(self.admin_clients("nova").services.list.return_value,
+                         result)
+        self.admin_clients("nova").services.list.assert_called_once_with(
+            host, binary)
+        self._test_atomic_action_timer(nova_scenario.atomic_actions(),
+                                       "nova.list_services")
+
+    def test__list_flavor_access(self):
+        nova_scenario = utils.NovaScenario()
+        result = nova_scenario._list_flavor_access("foo_id")
+        self.assertEqual(
+            self.admin_clients("nova").flavor_access.list.return_value,
+            result)
+        self.admin_clients("nova").flavor_access.list.assert_called_once_with(
+            flavor="foo_id")
+        self._test_atomic_action_timer(nova_scenario.atomic_actions(),
+                                       "nova.list_flavor_access")
+
+    def test__create_flavor(self):
+        nova_scenario = utils.NovaScenario()
+        random_name = "random_name"
+        nova_scenario.generate_random_name = mock.Mock(
+            return_value=random_name)
+        result = nova_scenario._create_flavor(500, 1, 1,
+                                              fakearg="fakearg")
+        self.assertEqual(
+            self.admin_clients("nova").flavors.create.return_value,
+            result)
+        self.admin_clients("nova").flavors.create.assert_called_once_with(
+            random_name, 500, 1, 1, fakearg="fakearg")
+        self._test_atomic_action_timer(nova_scenario.atomic_actions(),
+                                       "nova.create_flavor")
+
+    def test__update_server(self):
+        server = mock.Mock()
+        nova_scenario = utils.NovaScenario()
+        nova_scenario.generate_random_name = mock.Mock(
+            return_value="new_name")
+        server.update = mock.Mock()
+
+        result = nova_scenario._update_server(server)
+        self.assertEqual(result, server.update.return_value)
+        nova_scenario.generate_random_name.assert_called_once_with()
+        server.update.assert_called_once_with(name="new_name")
+
+        nova_scenario.generate_random_name.reset_mock()
+        server.update.reset_mock()
+
+        result = nova_scenario._update_server(server,
+                                              description="desp")
+        self.assertEqual(result, server.update.return_value)
+        nova_scenario.generate_random_name.assert_called_once_with()
+        server.update.assert_called_once_with(name="new_name",
+                                              description="desp")
+
+        self._test_atomic_action_timer(nova_scenario.atomic_actions(),
+                                       "nova.update_server")
