@@ -13,14 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ddt
 import mock
 
-from rally.common import logging
 from rally import exceptions
 from rally.plugins.openstack.scenarios.vm import vmtasks
 from tests.unit import test
 
 
+@ddt.ddt
 class VMTasksTestCase(test.ScenarioTestCase):
 
     def setUp(self):
@@ -36,45 +37,16 @@ class VMTasksTestCase(test.ScenarioTestCase):
         self.scenario._create_volume = mock.Mock(
             return_value=mock.Mock(id="foo_volume"))
         self.scenario._run_command = mock.MagicMock(
-            return_value=(0, "\"foo_out\"", "foo_err"))
+            return_value=(0, "{\"foo\": 42}", "foo_err"))
+        self.scenario.add_output = mock.Mock()
 
     def test_boot_runcommand_delete(self):
-        with logging.LogCatcher(logging.LOG) as catcher:
-            self.scenario.boot_runcommand_delete(
-                "foo_image", "foo_flavor",
-                script="foo_script", interpreter="foo_interpreter",
-                username="foo_username",
-                password="foo_password",
-                use_floating_ip="use_fip",
-                floating_network="ext_network",
-                force_delete="foo_force",
-                volume_args={"size": 16},
-                foo_arg="foo_value")
-
-        catcher.assertInLogs(
-            "Use `command' argument instead (args `script', `interpreter' "
-            "deprecated in Rally v0.0.5)")
-
-        self.scenario._create_volume.assert_called_once_with(
-            16, imageRef=None)
-        self.scenario._boot_server_with_fip.assert_called_once_with(
-            "foo_image", "foo_flavor", key_name="keypair_name",
-            use_floating_ip="use_fip", floating_network="ext_network",
-            block_device_mapping={"vdrally": "foo_volume:::1"},
-            foo_arg="foo_value")
-
-        self.scenario._wait_for_ping.assert_called_once_with("foo_ip")
-        self.scenario._run_command.assert_called_once_with(
-            "foo_ip", 22, "foo_username", "foo_password",
-            command={"script_file": "foo_script",
-                     "interpreter": "foo_interpreter"})
-        self.scenario._delete_server_with_fip.assert_called_once_with(
-            "foo_server", self.ip, force_delete="foo_force")
-
-    def test_boot_runcommand_delete_command(self):
+        self.scenario._run_command = mock.MagicMock(
+            return_value=(0, "{\"foo\": 42}", "foo_err"))
         self.scenario.boot_runcommand_delete(
             "foo_image", "foo_flavor",
-            command={"remote_path": "foo"},
+            command={"script_file": "foo_script",
+                     "interpreter": "foo_interpreter"},
             username="foo_username",
             password="foo_password",
             use_floating_ip="use_fip",
@@ -91,21 +63,68 @@ class VMTasksTestCase(test.ScenarioTestCase):
             block_device_mapping={"vdrally": "foo_volume:::1"},
             foo_arg="foo_value")
 
+        self.scenario._wait_for_ping.assert_called_once_with("foo_ip")
         self.scenario._run_command.assert_called_once_with(
             "foo_ip", 22, "foo_username", "foo_password",
-            command={"remote_path": "foo"})
+            command={"script_file": "foo_script",
+                     "interpreter": "foo_interpreter"})
         self.scenario._delete_server_with_fip.assert_called_once_with(
             "foo_server", self.ip, force_delete="foo_force")
+        self.scenario.add_output.assert_called_once_with(
+            additive={"title": "Command output", "chart_plugin": "Lines",
+                      "data": [["foo", 42.0]]})
 
-    def test_boot_runcommand_delete_script_fails(self):
-        self.scenario._run_command = mock.MagicMock(
-            return_value=(1, "\"foo_out\"", "foo_err"))
-        self.assertRaises(exceptions.ScriptError,
-                          self.scenario.boot_runcommand_delete,
-                          "foo_image", "foo_flavor", "foo_interpreter",
-                          "foo_script", "foo_username")
-        self.scenario._delete_server_with_fip.assert_called_once_with(
-            "foo_server", self.ip, force_delete=False)
+    @ddt.data(
+        {"output": (0, "", ""), "raises": exceptions.ScriptError},
+        {"output": (0, "{\"foo\": 42}", ""),
+         "expected": [{"additive": {"chart_plugin": "Lines",
+                                    "data": [["foo", 42.0]],
+                                    "title": "Command output"}}]},
+        {"output": (1, "{\"foo\": 42}", ""), "raises": exceptions.ScriptError},
+        {"output": ("", 1, ""), "raises": TypeError},
+        {"output": (0, "{\"additive\": [1, 2]}", ""),
+         "expected": [{"additive": 1}, {"additive": 2}]},
+        {"output": (0, "{\"complete\": [3, 4]}", ""),
+         "expected": [{"complete": 3}, {"complete": 4}]},
+        {"output": (0, "{\"additive\": [1, 2], \"complete\": [3, 4]}", ""),
+         "expected": [{"additive": 1}, {"additive": 2},
+                      {"complete": 3}, {"complete": 4}]}
+    )
+    @ddt.unpack
+    def test_boot_runcommand_delete_add_output(self, output,
+                                               expected=None, raises=None):
+        self.scenario._run_command.return_value = output
+        kwargs = {"command": {"remote_path": "foo"},
+                  "username": "foo_username",
+                  "password": "foo_password",
+                  "use_floating_ip": "use_fip",
+                  "floating_network": "ext_network",
+                  "force_delete": "foo_force",
+                  "volume_args": {"size": 16},
+                  "foo_arg": "foo_value"}
+        if raises:
+            self.assertRaises(raises, self.scenario.boot_runcommand_delete,
+                              "foo_image", "foo_flavor", **kwargs)
+            self.assertFalse(self.scenario.add_output.called)
+        else:
+            self.scenario.boot_runcommand_delete("foo_image", "foo_flavor",
+                                                 **kwargs)
+            calls = [mock.call(**kw) for kw in expected]
+            self.scenario.add_output.assert_has_calls(calls, any_order=True)
+
+            self.scenario._create_volume.assert_called_once_with(
+                16, imageRef=None)
+            self.scenario._boot_server_with_fip.assert_called_once_with(
+                "foo_image", "foo_flavor", key_name="keypair_name",
+                use_floating_ip="use_fip", floating_network="ext_network",
+                block_device_mapping={"vdrally": "foo_volume:::1"},
+                foo_arg="foo_value")
+
+            self.scenario._run_command.assert_called_once_with(
+                "foo_ip", 22, "foo_username", "foo_password",
+                command={"remote_path": "foo"})
+            self.scenario._delete_server_with_fip.assert_called_once_with(
+                "foo_server", self.ip, force_delete="foo_force")
 
     def test_boot_runcommand_delete_command_timeouts(self):
         self.scenario._run_command.side_effect = exceptions.SSHTimeout()
@@ -115,6 +134,7 @@ class VMTasksTestCase(test.ScenarioTestCase):
                           "foo_script", "foo_username")
         self.scenario._delete_server_with_fip.assert_called_once_with(
             "foo_server", self.ip, force_delete=False)
+        self.assertFalse(self.scenario.add_output.called)
 
     def test_boot_runcommand_delete_ping_wait_timeouts(self):
         self.scenario._wait_for_ping.side_effect = exceptions.TimeoutException(
@@ -136,6 +156,7 @@ class VMTasksTestCase(test.ScenarioTestCase):
 
         self.scenario._delete_server_with_fip.assert_called_once_with(
             "foo_server", self.ip, force_delete=False)
+        self.assertFalse(self.scenario.add_output.called)
 
     @mock.patch("rally.plugins.openstack.scenarios.vm.vmtasks.json")
     def test_boot_runcommand_delete_json_fails(self, mock_json):
@@ -146,6 +167,7 @@ class VMTasksTestCase(test.ScenarioTestCase):
                           "foo_script", "foo_username")
         self.scenario._delete_server_with_fip.assert_called_once_with(
             "foo_server", self.ip, force_delete=False)
+        self.assertFalse(self.scenario.add_output.called)
 
     def test_boot_runcommand_delete_custom_image(self):
         context = {
