@@ -14,6 +14,7 @@
 #    under the License.
 
 import traceback
+import uuid
 
 import mock
 
@@ -30,12 +31,13 @@ class ScenarioConfigureTestCase(test.TestCase):
 
     def test_configure(self):
 
-        @scenario.configure("test_configure", "testing")
+        @scenario.configure("fooscenario.name", "testing")
         def some_func():
             pass
 
-        self.assertEqual("test_configure", some_func.get_name())
+        self.assertEqual("fooscenario.name", some_func.get_name())
         self.assertEqual("testing", some_func.get_namespace())
+        self.assertFalse(some_func.is_classbased)
         some_func.unregister()
 
     def test_configure_default_name(self):
@@ -47,6 +49,7 @@ class ScenarioConfigureTestCase(test.TestCase):
         self.assertIsNone(some_func._meta_get("name"))
         self.assertEqual("testing", some_func.get_namespace())
         self.assertEqual({"any": 42}, some_func._meta_get("default_context"))
+        self.assertFalse(some_func.is_classbased)
         some_func.unregister()
 
     def test_configure_cls(self):
@@ -62,8 +65,20 @@ class ScenarioConfigureTestCase(test.TestCase):
         self.assertEqual("any", ScenarioPluginCls.some.get_namespace())
         self.assertEqual({"any": 43},
                          ScenarioPluginCls.some._meta_get("default_context"))
+        self.assertFalse(ScenarioPluginCls.some.is_classbased)
         ScenarioPluginCls.some.unregister()
 
+    def test_configure_classbased(self):
+
+        @scenario.configure(name="fooscenario.name", namespace="testing")
+        class SomeScenario(scenario.Scenario):
+            def run(self):
+                pass
+
+        self.assertEqual("fooscenario.name", SomeScenario.get_name())
+        self.assertTrue(SomeScenario.is_classbased)
+        SomeScenario.unregister()
+
 
 class ScenarioTestCase(test.TestCase):
 
@@ -106,8 +121,7 @@ class ScenarioTestCase(test.TestCase):
                           scenario.Scenario._validate_helper,
                           validators, clients, args, "fake_uuid")
 
-    @mock.patch("rally.task.scenario.Scenario.get")
-    def test_validate__no_validators(self, mock_scenario_get):
+    def test_validate__no_validators(self):
 
         class Testing(fakes.FakeScenario):
 
@@ -115,17 +129,12 @@ class ScenarioTestCase(test.TestCase):
             def validate__no_validators(self):
                 pass
 
-        mock_scenario_get.return_value = Testing.validate__no_validators
         scenario.Scenario.validate("Testing.validate__no_validators",
                                    {"a": 1, "b": 2})
-        mock_scenario_get.assert_called_once_with(
-            "Testing.validate__no_validators")
         Testing.validate__no_validators.unregister()
 
     @mock.patch("rally.task.scenario.Scenario._validate_helper")
-    @mock.patch("rally.task.scenario.Scenario.get")
-    def test_validate__admin_validators(self, mock_scenario_get,
-                                        mock_scenario__validate_helper):
+    def test_validate__admin_validators(self, mock_scenario__validate_helper):
 
         class Testing(fakes.FakeScenario):
 
@@ -133,8 +142,6 @@ class ScenarioTestCase(test.TestCase):
             def validate_admin_validators(self):
                 pass
 
-        mock_scenario_get.return_value = Testing.validate_admin_validators
-
         validators = [mock.MagicMock(), mock.MagicMock()]
         for validator in validators:
             validator.permission = consts.EndpointPermission.ADMIN
@@ -151,9 +158,7 @@ class ScenarioTestCase(test.TestCase):
         Testing.validate_admin_validators.unregister()
 
     @mock.patch("rally.task.scenario.Scenario._validate_helper")
-    @mock.patch("rally.task.scenario.Scenario.get")
-    def test_validate_user_validators(self, mock_scenario_get,
-                                      mock_scenario__validate_helper):
+    def test_validate_user_validators(self, mock_scenario__validate_helper):
 
         class Testing(fakes.FakeScenario):
 
@@ -161,8 +166,6 @@ class ScenarioTestCase(test.TestCase):
             def validate_user_validators(self):
                 pass
 
-        mock_scenario_get.return_value = Testing.validate_user_validators
-
         validators = [mock.MagicMock(), mock.MagicMock()]
         for validator in validators:
             validator.permission = consts.EndpointPermission.USER
@@ -179,6 +182,48 @@ class ScenarioTestCase(test.TestCase):
 
         Testing.validate_user_validators.unregister()
 
+    def test__validate_scenario_args(self):
+
+        class Testing(fakes.FakeScenario):
+            @scenario.configure()
+            def fake_scenario_to_validate_scenario_args(self, arg1, arg2,
+                                                        arg3=None):
+                pass
+
+        name = "Testing.fake_scenario_to_validate_scenario_args"
+        scen = scenario.Scenario.get(name)
+
+        # check case when argument is missed
+        e = self.assertRaises(exceptions.InvalidArgumentsException,
+                              scenario.Scenario._validate_scenario_args,
+                              scen, name, {"args": {"arg1": 3}})
+        self.assertIn("Argument(s) 'arg2' should be specified in task config.",
+                      e.format_message())
+
+        # check case when one argument is redundant
+        e = self.assertRaises(exceptions.InvalidArgumentsException,
+                              scenario.Scenario._validate_scenario_args,
+                              scen, name,
+                              {"args": {"arg1": 1, "arg2": 2, "arg4": 4}})
+
+        self.assertIn("Unexpected argument(s) found ['arg4']",
+                      e.format_message())
+
+    def test__validate_scenario_args_with_class_based_scenario(self):
+        name = "%s.need_dot" % uuid.uuid4()
+
+        @scenario.configure(name=name)
+        class Testing(scenario.Scenario):
+            def run(self, arg):
+                pass
+
+        e = self.assertRaises(exceptions.InvalidArgumentsException,
+                              scenario.Scenario._validate_scenario_args,
+                              Testing, name, {})
+
+        self.assertIn("Argument(s) 'arg' should be specified in task config.",
+                      e.format_message())
+
     def test_sleep_between_invalid_args(self):
         self.assertRaises(exceptions.InvalidArgumentsException,
                           scenario.Scenario().sleep_between, 15, 5)
