@@ -252,9 +252,9 @@ class ValidatorsTestCase(test.TestCase):
         self.assertTrue(result[0].is_valid, result[0].msg)
         self.assertEqual(result[1], image)
 
-    @mock.patch(MODULE + "types.ImageResourceType.transform",
+    @mock.patch(MODULE + "openstack_types.GlanceImage.transform",
                 return_value="image_id")
-    def test__get_validated_image(self, mock_image_resource_type_transform):
+    def test__get_validated_image(self, mock_glance_image_transform):
         clients = mock.MagicMock()
         clients.glance().images.get().to_dict.return_value = {
             "image": "image_id"}
@@ -267,21 +267,21 @@ class ValidatorsTestCase(test.TestCase):
         self.assertEqual({"image": "image_id", "min_disk": 0,
                           "min_ram": 0, "size": 0},
                          result[1])
-        mock_image_resource_type_transform.assert_called_once_with(
+        mock_glance_image_transform.assert_called_once_with(
             clients=clients, resource_config="test")
         clients.glance().images.get.assert_called_with(image="image_id")
 
-    @mock.patch(MODULE + "types.ImageResourceType.transform",
+    @mock.patch(MODULE + "openstack_types.GlanceImage.transform",
                 side_effect=exceptions.InvalidScenarioArgument)
     def test__get_validated_image_transform_error(
-            self, mock_image_resource_type_transform):
+            self, mock_glance_image_transform):
         result = validation._get_validated_image({"args": {"a": "test"}},
                                                  None, "a")
         self.assertFalse(result[0].is_valid, result[0].msg)
 
-    @mock.patch(MODULE + "types.ImageResourceType.transform")
+    @mock.patch(MODULE + "openstack_types.GlanceImage.transform")
     def test__get_validated_image_not_found(
-            self, mock_image_resource_type_transform):
+            self, mock_glance_image_transform):
         clients = mock.MagicMock()
         clients.glance().images.get().to_dict.side_effect = (
             glance_exc.HTTPNotFound(""))
@@ -293,10 +293,10 @@ class ValidatorsTestCase(test.TestCase):
         result = validation._get_validated_flavor({}, None, "non_existing")
         self.assertFalse(result[0].is_valid, result[0].msg)
 
-    @mock.patch(MODULE + "types.FlavorResourceType.transform",
+    @mock.patch(MODULE + "openstack_types.Flavor.transform",
                 return_value="flavor_id")
     def test__get_validated_flavor(
-            self, mock_flavor_resource_type_transform):
+            self, mock_flavor_transform):
         clients = mock.MagicMock()
         clients.nova().flavors.get.return_value = "flavor"
 
@@ -304,30 +304,30 @@ class ValidatorsTestCase(test.TestCase):
                                                   clients, "a")
         self.assertTrue(result[0].is_valid, result[0].msg)
         self.assertEqual(result[1], "flavor")
-        mock_flavor_resource_type_transform.assert_called_once_with(
+        mock_flavor_transform.assert_called_once_with(
             clients=clients, resource_config="test")
         clients.nova().flavors.get.assert_called_once_with(flavor="flavor_id")
 
-    @mock.patch(MODULE + "types.FlavorResourceType.transform",
+    @mock.patch(MODULE + "openstack_types.Flavor.transform",
                 side_effect=exceptions.InvalidScenarioArgument)
     def test__get_validated_flavor_transform_error(
-            self, mock_flavor_resource_type_transform):
+            self, mock_flavor_transform):
         result = validation._get_validated_flavor({"args": {"a": "test"}},
                                                   None, "a")
         self.assertFalse(result[0].is_valid, result[0].msg)
 
-    @mock.patch(MODULE + "types.FlavorResourceType.transform")
+    @mock.patch(MODULE + "openstack_types.Flavor.transform")
     def test__get_validated_flavor_not_found(
-            self, mock_flavor_resource_type_transform):
+            self, mock_flavor_transform):
         clients = mock.MagicMock()
         clients.nova().flavors.get.side_effect = nova_exc.NotFound("")
         result = validation._get_validated_flavor({"args": {"a": "test"}},
                                                   clients, "a")
         self.assertFalse(result[0].is_valid, result[0].msg)
 
-    @mock.patch(MODULE + "types.FlavorResourceType.transform")
+    @mock.patch(MODULE + "openstack_types.Flavor.transform")
     def test__get_validated_flavor_from_context(
-            self, mock_flavor_resource_type_transform):
+            self, mock_flavor_transform):
         clients = mock.MagicMock()
         clients.nova().flavors.get.side_effect = nova_exc.NotFound("")
         config = {
@@ -342,9 +342,9 @@ class ValidatorsTestCase(test.TestCase):
         result = validation._get_validated_flavor(config, clients, "flavor")
         self.assertTrue(result[0].is_valid, result[0].msg)
 
-    @mock.patch(MODULE + "types.FlavorResourceType.transform")
+    @mock.patch(MODULE + "openstack_types.Flavor.transform")
     def test__get_validated_flavor_from_context_failed(
-            self, mock_flavor_resource_type_transform):
+            self, mock_flavor_transform):
         clients = mock.MagicMock()
         clients.nova().flavors.get.side_effect = nova_exc.NotFound("")
         config = {
@@ -423,9 +423,9 @@ class ValidatorsTestCase(test.TestCase):
         mock__get_validated_flavor.return_value = (success, flavor)
         mock__get_validated_image.return_value = (success, image)
 
+        # test flavor.disk None
         validator = self._unwrap_validator(validation.image_valid_on_flavor,
                                            "flavor", "image")
-        # test ram
         flavor.disk = None
         flavor.ram = 2
         image["min_ram"] = 4
@@ -435,7 +435,21 @@ class ValidatorsTestCase(test.TestCase):
         result = validator(None, None, None)
         self.assertTrue(result.is_valid, result.msg)
 
-        # test disk (flavor.disk not None)
+        # test validate_disk false
+        validator = self._unwrap_validator(validation.image_valid_on_flavor,
+                                           "flavor", "image", False)
+        flavor.disk = 1
+        flavor.ram = 2
+        image["min_ram"] = 4
+        result = validator(None, None, None)
+        self.assertFalse(result.is_valid, result.msg)
+        image["min_ram"] = 1
+        result = validator(None, None, None)
+        self.assertTrue(result.is_valid, result.msg)
+
+        # test validate_disk true and flavor.disk not None
+        validator = self._unwrap_validator(validation.image_valid_on_flavor,
+                                           "flavor", "image")
         image["size"] = 2
         image["min_disk"] = 0
         flavor.disk = 5.0 / (1024 ** 3)
@@ -449,11 +463,11 @@ class ValidatorsTestCase(test.TestCase):
         result = validator(None, None, None)
         self.assertFalse(result.is_valid, result.msg)
 
-    @mock.patch(MODULE + "types.FlavorResourceType.transform")
+    @mock.patch(MODULE + "openstack_types.Flavor.transform")
     @mock.patch(MODULE + "_get_validated_image")
     def test_image_valid_on_flavor_context(
             self, mock__get_validated_image,
-            mock_flavor_resource_type_transform):
+            mock_flavor_transform):
         clients = mock.MagicMock()
         clients.nova().flavors.get.side_effect = nova_exc.NotFound("")
 
@@ -883,3 +897,80 @@ class ValidatorsTestCase(test.TestCase):
             self.assertEqual("Heat template validation failed on fake_path1."
                              " Original error message: fake_msg.", result.msg)
             self.assertFalse(result.is_valid)
+
+    def _get_keystone_v2_mock_client(self):
+        keystone = mock.Mock()
+        del keystone.projects
+        keystone.tenants = mock.Mock()
+        return keystone
+
+    def _get_keystone_v3_mock_client(self):
+        keystone = mock.Mock()
+        del keystone.tenants
+        keystone.projects = mock.Mock()
+        return keystone
+
+    def test_required_api_versions_keystonev2(self):
+        validator = self._unwrap_validator(
+            validation.required_api_versions, component="keystone",
+            versions=[2.0])
+        clients = mock.MagicMock()
+        clients.keystone.return_value = self._get_keystone_v3_mock_client()
+        self.assertFalse(validator({}, clients, None).is_valid)
+
+        clients.keystone.return_value = self._get_keystone_v2_mock_client()
+        self.assertTrue(validator({}, clients, None).is_valid)
+
+    def test_required_api_versions_keystonev3(self):
+        validator = self._unwrap_validator(
+            validation.required_api_versions, component="keystone",
+            versions=[3])
+        clients = mock.MagicMock()
+
+        clients.keystone.return_value = self._get_keystone_v2_mock_client()
+        self.assertFalse(validator({}, clients, None).is_valid)
+
+        clients.keystone.return_value = self._get_keystone_v3_mock_client()
+        self.assertTrue(validator({}, clients, None).is_valid)
+
+    def test_required_api_versions_keystone_all_versions(self):
+        validator = self._unwrap_validator(
+            validation.required_api_versions, component="keystone",
+            versions=[2.0, 3])
+        clients = mock.MagicMock()
+
+        clients.keystone.return_value = self._get_keystone_v3_mock_client()
+        self.assertTrue(validator({}, clients, None).is_valid)
+
+        clients.keystone.return_value = self._get_keystone_v2_mock_client()
+        self.assertTrue(validator({}, clients, None).is_valid)
+
+    @ddt.data({"nova_version": 2, "required_versions": [2], "valid": True},
+              {"nova_version": 3, "required_versions": [2], "valid": False},
+              {"nova_version": None, "required_versions": [2], "valid": False},
+              {"nova_version": 2, "required_versions": [2, 3], "valid": True},
+              {"nova_version": 4, "required_versions": [2, 3], "valid": False})
+    @ddt.unpack
+    def test_required_api_versions_choose_version(self, nova_version=None,
+                                                  required_versions=(2,),
+                                                  valid=False):
+        validator = self._unwrap_validator(
+            validation.required_api_versions, component="nova",
+            versions=required_versions)
+        clients = mock.MagicMock()
+        clients.nova.choose_version.return_value = nova_version
+        self.assertEqual(validator({}, clients, None).is_valid,
+                         valid)
+
+    @ddt.data({"required_version": 2, "valid": True},
+              {"required_version": 3, "valid": False})
+    @ddt.unpack
+    def test_required_api_versions_context(self, required_version=None,
+                                           valid=False):
+        validator = self._unwrap_validator(
+            validation.required_api_versions, component="nova",
+            versions=[required_version])
+        clients = mock.MagicMock()
+        config = {"context": {"api_versions": {"nova": {"version": 2}}}}
+        self.assertEqual(validator(config, clients, None).is_valid,
+                         valid)
