@@ -25,6 +25,7 @@ import mock
 from rally import api
 from rally.common import objects
 from rally import consts
+from rally.deployment import engine
 from rally import exceptions
 from tests.unit import fakes
 from tests.unit import test
@@ -412,38 +413,18 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
         super(VerificationAPITestCase, self).setUp()
         self.tempest = mock.Mock()
 
+    @mock.patch("os.path.exists", return_value=True)
     @mock.patch("rally.common.objects.Deployment.get")
     @mock.patch("rally.api.objects.Verification")
     @mock.patch("rally.verification.tempest.tempest.Tempest")
     def test_verify(self, mock_tempest, mock_verification,
-                    mock_deployment_get):
+                    mock_deployment_get, mock_exists):
         mock_deployment_get.return_value = {"uuid": self.deployment_uuid}
-
         mock_tempest.return_value = self.tempest
-        self.tempest.is_installed.return_value = True
         api.Verification.verify(
             self.deployment_uuid, set_name="smoke",
             regex=None, tests_file=None, tempest_config=None)
 
-        self.tempest.is_installed.assert_called_once_with()
-        self.tempest.verify.assert_called_once_with(
-            set_name="smoke", regex=None, tests_file=None,
-            expected_failures=None, concur=0, failing=False)
-
-    @mock.patch("rally.api.objects.Deployment.get")
-    @mock.patch("rally.api.objects.Verification")
-    @mock.patch("rally.verification.tempest.tempest.Tempest")
-    def test_verify_tempest_not_installed(self, mock_tempest,
-                                          mock_verification,
-                                          mock_deployment_get):
-        mock_deployment_get.return_value = {"uuid": self.deployment_uuid}
-        mock_tempest.return_value = self.tempest
-        self.tempest.is_installed.return_value = False
-        api.Verification.verify(
-            self.deployment_uuid, set_name="smoke",
-            regex=None, tests_file=None, tempest_config=None)
-        self.tempest.is_installed.assert_called_once_with()
-        self.tempest.install.assert_called_once_with()
         self.tempest.verify.assert_called_once_with(
             set_name="smoke", regex=None, tests_file=None,
             expected_failures=None, concur=0, failing=False)
@@ -456,7 +437,6 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
                                          mock_deployment_get, mock_exists):
         mock_deployment_get.return_value = {"uuid": self.deployment_uuid}
         mock_tempest.return_value = self.tempest
-        self.tempest.is_installed.return_value = True
         tests_file = "/path/to/tests/file"
         api.Verification.verify(
             self.deployment_uuid, set_name="", regex=None,
@@ -466,6 +446,16 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
             set_name="", regex=None, tests_file=tests_file,
             expected_failures=None, concur=0, failing=False)
 
+    @mock.patch("rally.api.objects.Deployment.get")
+    @mock.patch("rally.api.objects.Verification")
+    def test_verify_no_tempest_tree_exists(self, mock_verification,
+                                           mock_deployment_get):
+        mock_deployment_get.return_value = {"uuid": self.deployment_uuid}
+        self.assertRaises(
+            exceptions.NotFoundException, api.Verification.verify,
+            self.deployment_uuid, set_name="smoke", regex=None,
+            tests_file=None, tempest_config=None)
+
     @mock.patch("rally.common.objects.Deployment.get")
     @mock.patch("rally.api.objects.Verification")
     @mock.patch("rally.verification.tempest.tempest.Tempest")
@@ -495,31 +485,41 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
         api.Verification.uninstall_tempest(self.deployment_uuid)
         self.tempest.uninstall.assert_called_once_with()
 
-    @mock.patch("tempfile.gettempdir")
-    @mock.patch("shutil.move")
-    @mock.patch("shutil.copy2")
     @mock.patch("rally.api.objects.Deployment.get")
     @mock.patch("rally.api.tempest.Tempest")
-    def test_reinstall_tempest(self, mock_tempest, mock_deployment_get,
-                               mock_copy2, mock_move, mock_gettempdir):
-
+    def test_reinstall_tempest(self, mock_tempest, mock_deployment_get):
         fake_source = "fake__source"
-        fake_conf = "/path/to/fake_conf"
-        fake_tmpdir = "/fake/tmp/path/to/dir"
-        tmp_file = os.path.join(fake_tmpdir, "fake_conf")
-        self.tempest.config_file = fake_conf
         mock_tempest.return_value = self.tempest
-        mock_gettempdir.return_value = fake_tmpdir
         api.Verification.reinstall_tempest(self.deployment_uuid,
                                            source=fake_source)
         self.tempest.uninstall.assert_called_once_with()
-        mock_copy2.assert_called_once_with(fake_conf, tmp_file)
         self.tempest.install.assert_called_once_with()
-        mock_move.assert_called_once_with(tmp_file, fake_conf)
 
+    @mock.patch("os.path.exists", return_value=True)
     @mock.patch("rally.common.objects.Deployment.get")
     @mock.patch("rally.verification.tempest.tempest.Tempest")
-    def test_discover_tests(self, mock_tempest, mock_deployment_get):
+    def test_install_tempest_plugin_from_url(
+            self, mock_tempest, mock_deployment_get, mock_exists):
+        mock_tempest.return_value = self.tempest
+        api.Verification.install_tempest_plugin(self.deployment_uuid,
+                                                "https://fake/plugin")
+        self.tempest.install_plugin.assert_called_once_with()
+
+    @mock.patch("os.path.exists", return_value=True)
+    @mock.patch("rally.common.objects.Deployment.get")
+    @mock.patch("rally.verification.tempest.tempest.Tempest")
+    def test_install_tempest_plugin_from_path(
+            self, mock_tempest, mock_deployment_get, mock_exists):
+        mock_tempest.return_value = self.tempest
+        api.Verification.install_tempest_plugin(self.deployment_uuid,
+                                                "/tmp/fake/plugin")
+        self.tempest.install_plugin.assert_called_once_with()
+
+    @mock.patch("os.path.exists", return_value=True)
+    @mock.patch("rally.common.objects.Deployment.get")
+    @mock.patch("rally.verification.tempest.tempest.Tempest")
+    def test_discover_tests(
+            self, mock_tempest, mock_deployment_get, mock_exists):
         mock_tempest.return_value = self.tempest
         api.Verification.discover_tests(self.deployment_uuid, "some_pattern")
         self.tempest.discover_tests.assert_called_once_with("some_pattern")
@@ -592,3 +592,25 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
         retval = api.Verification.get("fake_id")
         self.assertEqual(mock_verification_get.return_value, retval)
         mock_verification_get.assert_called_once_with("fake_id")
+
+    @mock.patch("rally.common.objects.deploy.db.deployment_delete")
+    @mock.patch("rally.common.objects.deploy.db.deployment_update")
+    @mock.patch("rally.common.objects.deploy.db.deployment_get")
+    def test_destroy_invalid_deployment_type(self, mock_deployment_get,
+                                             mock_deployment_update,
+                                             mock_deployment_delete):
+        with mock.patch.dict(self.deployment["config"],
+                             {"type": "InvalidDeploymentType"}):
+            deployment = mock.Mock()
+            deployment.update_status = lambda x: x
+            deployment.__getitem__ = lambda _self, key: self.deployment[key]
+            self.assertRaises(exceptions.PluginNotFound,
+                              engine.Engine.get_engine,
+                              self.deployment["config"]["type"],
+                              deployment)
+            mock_deployment_get.return_value = self.deployment
+            mock_deployment_update.return_value = self.deployment
+            api.Deployment.destroy(self.deployment_uuid)
+            mock_deployment_get.assert_called_once_with(self.deployment_uuid)
+            mock_deployment_delete.assert_called_once_with(
+                self.deployment_uuid)
