@@ -297,6 +297,19 @@ class OSClientsTestCase(test.TestCase):
             auth=fake_auth, verify=not self.credential.insecure,
             timeout=cfg.CONF.openstack_client_http_timeout)
 
+    @mock.patch("keystoneclient.session.Session")
+    def test_get_session_with_ca(self, mock_session):
+        # Use DummyClient since if not the abc meta kicks in
+        osc = DummyClient(self.credential, {}, {})
+
+        self.credential.cacert = "/fake/ca"
+        fake_auth = mock.Mock()
+        osc._get_session(auth=fake_auth)
+
+        mock_session.assert_called_once_with(
+            auth=fake_auth, verify="/fake/ca",
+            timeout=cfg.CONF.openstack_client_http_timeout)
+
     def test_keystone(self):
         self.assertNotIn("keystone", self.clients.cache)
         client = self.clients.keystone()
@@ -493,6 +506,24 @@ class OSClientsTestCase(test.TestCase):
             self.assertEqual(fake_ceilometer,
                              self.clients.cache["ceilometer"])
 
+    def test_gnocchi(self):
+        fake_gnocchi = fakes.FakeGnocchiClient()
+        mock_gnocchi = mock.MagicMock()
+        mock_gnocchi.client.Client.return_value = fake_gnocchi
+        mock_keystoneauth1 = mock.MagicMock()
+        self.assertNotIn("gnocchi", self.clients.cache)
+        with mock.patch.dict("sys.modules",
+                             {"gnocchiclient": mock_gnocchi,
+                              "keystoneauth1": mock_keystoneauth1}):
+            client = self.clients.gnocchi()
+
+            self.assertEqual(fake_gnocchi, client)
+            kw = {"version": "1",
+                  "session": mock_keystoneauth1.session.Session(),
+                  "service_type": "metric"}
+            mock_gnocchi.client.Client.assert_called_once_with(**kw)
+            self.assertEqual(fake_gnocchi, self.clients.cache["gnocchi"])
+
     def test_monasca(self):
         fake_monasca = fakes.FakeMonascaClient()
         mock_monasca = mock.MagicMock()
@@ -554,10 +585,12 @@ class OSClientsTestCase(test.TestCase):
             self.assertEqual(fake_sahara, client)
             kw = {
                 "service_type": "data-processing",
+                "endpoint_type": self.credential.endpoint_type,
                 "insecure": False,
                 "username": self.credential.username,
                 "api_key": self.credential.password,
                 "project_name": self.credential.tenant_name,
+                "cacert": self.credential.cacert,
                 "auth_url": self.credential.auth_url
             }
             mock_sahara.client.Client.assert_called_once_with(1.1, **kw)
@@ -653,9 +686,7 @@ class OSClientsTestCase(test.TestCase):
                   "insecure": False,
                   "cacert": None,
                   "user": self.credential.username,
-                  "key": self.credential.password,
                   "tenant_name": self.credential.tenant_name,
-                  "authurl": self.credential.auth_url
                   }
             mock_swift.client.Connection.assert_called_once_with(**kw)
             self.assertEqual(self.clients.cache["swift"], fake_swift)
@@ -783,3 +814,70 @@ class OSClientsTestCase(test.TestCase):
                 interface=consts.EndpointType.PUBLIC,
                 session=self.fake_keystone.session)
             self.assertEqual(fake_cue, self.clients.cache["cue"])
+
+    def test_senlin(self):
+        mock_senlin = mock.MagicMock()
+        self.assertNotIn("senlin", self.clients.cache)
+        with mock.patch.dict("sys.modules", {"senlinclient": mock_senlin}):
+            client = self.clients.senlin()
+            self.assertEqual(mock_senlin.client.Client.return_value, client)
+            mock_senlin.client.Client.assert_called_once_with(
+                "1",
+                username=self.credential.username,
+                password=self.credential.password,
+                project_name=self.credential.tenant_name,
+                cert=self.credential.cacert,
+                auth_url=self.credential.auth_url)
+            self.assertEqual(
+                mock_senlin.client.Client.return_value,
+                self.clients.cache["senlin"])
+
+    @mock.patch("rally.osclients.Magnum._get_session")
+    def test_magnum(self, mock_magnum__get_session):
+        fake_magnum = fakes.FakeMagnumClient()
+        mock_magnum = mock.MagicMock()
+        mock_magnum.client.Client.return_value = fake_magnum
+
+        mock_magnum__get_session.return_value = self.fake_keystone.session
+
+        self.assertNotIn("magnum", self.clients.cache)
+        with mock.patch.dict("sys.modules", {"magnumclient": mock_magnum}):
+            client = self.clients.magnum()
+
+            self.assertEqual(fake_magnum, client)
+
+            self.service_catalog.url_for.assert_called_once_with(
+                service_type="container-infra",
+                endpoint_type=consts.EndpointType.PUBLIC,
+                region_name=self.credential.region_name)
+
+            mock_magnum.client.Client.assert_called_once_with(
+                interface=consts.EndpointType.PUBLIC,
+                session=self.fake_keystone.session)
+
+            self.assertEqual(fake_magnum, self.clients.cache["magnum"])
+
+    def test_watcher(self):
+        fake_watcher = fakes.FakeWatcherClient()
+        mock_watcher = mock.MagicMock()
+        mock_watcher.client.Client.return_value = fake_watcher
+        self.assertNotIn("watcher", self.clients.cache)
+        with mock.patch.dict("sys.modules", {"watcherclient": mock_watcher}):
+            client = self.clients.watcher()
+
+            self.assertEqual(fake_watcher, client)
+
+            self.service_catalog.url_for.assert_called_once_with(
+                service_type="infra-optim",
+                endpoint_type=consts.EndpointType.PUBLIC,
+                region_name=self.credential.region_name)
+
+            mock_watcher.client.Client.assert_called_once_with(
+                "1",
+                self.service_catalog.url_for.return_value,
+                token=self.fake_keystone.auth_token,
+                ca_file=None,
+                insecure=False,
+                timeout=180.0)
+
+            self.assertEqual(fake_watcher, self.clients.cache["watcher"])
