@@ -34,7 +34,7 @@ CREDS = {
         "username": "admin",
         "tenant_name": "admin",
         "password": "admin-12345",
-        "auth_url": "http://test/v2.0/",
+        "auth_url": "http://test:5000/v2.0/",
         "permission": "admin",
         "region_name": "test",
         "admin_domain_name": "Default",
@@ -61,26 +61,26 @@ class TempestConfigTestCase(test.TestCase):
     @mock.patch("os.rename")
     @mock.patch("six.moves.builtins.open", side_effect=mock.mock_open())
     @mock.patch("requests.get", return_value=mock.MagicMock(status_code=200))
-    def test__download_cirros_image_success(self, mock_get,
-                                            mock_open, mock_rename):
+    def test__download_image_success(self, mock_get,
+                                     mock_open, mock_rename):
         self.mock_isfile.return_value = False
-        self.tempest_conf._download_cirros_image()
+        self.tempest_conf._download_image()
         mock_get.assert_called_once_with(
-            CONF.image.cirros_img_url, stream=True)
+            CONF.tempest.img_url, stream=True)
 
     @mock.patch("requests.get")
     @ddt.data(404, 500)
-    def test__download_cirros_image_failure(self, status_code, mock_get):
+    def test__download_image_failure(self, status_code, mock_get):
         self.mock_isfile.return_value = False
         mock_get.return_value = mock.MagicMock(status_code=status_code)
         self.assertRaises(exceptions.TempestConfigCreationFailure,
-                          self.tempest_conf._download_cirros_image)
+                          self.tempest_conf._download_image)
 
     @mock.patch("requests.get", side_effect=requests.ConnectionError())
-    def test__download_cirros_image_connection_error(self, mock_requests_get):
+    def test__download_image_connection_error(self, mock_requests_get):
         self.mock_isfile.return_value = False
         self.assertRaises(exceptions.TempestConfigCreationFailure,
-                          self.tempest_conf._download_cirros_image)
+                          self.tempest_conf._download_image)
 
     @ddt.data({"publicURL": "test_url"},
               {"interface": "public", "url": "test_url"})
@@ -149,7 +149,7 @@ class TempestConfigTestCase(test.TestCase):
             ("admin_domain_name", CREDS["admin"]["admin_domain_name"]),
             ("region", CREDS["admin"]["region_name"]),
             ("auth_version", "v2"),
-            ("uri", CREDS["admin"]["auth_url"]),
+            ("uri", CREDS["admin"]["auth_url"][:-1]),
             ("uri_v3", CREDS["admin"]["auth_url"].replace("/v2.0/", "/v3")),
             ("disable_ssl_certificate_validation",
              str(CREDS["admin"]["https_insecure"])),
@@ -228,8 +228,8 @@ class TempestConfigTestCase(test.TestCase):
         self.tempest_conf._configure_object_storage()
 
         expected = (
-            ("operator_role", CONF.role.swift_operator_role),
-            ("reseller_admin_role", CONF.role.swift_reseller_admin_role))
+            ("operator_role", CONF.tempest.swift_operator_role),
+            ("reseller_admin_role", CONF.tempest.swift_reseller_admin_role))
         result = self.tempest_conf.conf.items("object-storage")
         for item in expected:
             self.assertIn(item, result)
@@ -238,8 +238,8 @@ class TempestConfigTestCase(test.TestCase):
         self.tempest_conf._configure_orchestration()
 
         expected = (
-            ("stack_owner_role", CONF.role.heat_stack_owner_role),
-            ("stack_user_role", CONF.role.heat_stack_user_role))
+            ("stack_owner_role", CONF.tempest.heat_stack_owner_role),
+            ("stack_user_role", CONF.tempest.heat_stack_user_role))
         result = self.tempest_conf.conf.items("orchestration")
         for item in expected:
             self.assertIn(item, result)
@@ -248,7 +248,7 @@ class TempestConfigTestCase(test.TestCase):
         self.tempest_conf._configure_scenario()
 
         image_name = parse.urlparse(
-            config.CONF.image.cirros_img_url).path.split("/")[-1]
+            config.CONF.tempest.img_url).path.split("/")[-1]
         expected = (("img_dir", self.tempest_conf.data_dir),
                     ("img_file", image_name))
         result = self.tempest_conf.conf.items("scenario")
@@ -256,14 +256,14 @@ class TempestConfigTestCase(test.TestCase):
             self.assertIn(item, result)
 
     def test__configure_service_available(self):
-        available_services = ("nova", "cinder", "glance", "sahara")
+        available_services = ("aodh", "nova", "cinder", "glance", "sahara")
         self.tempest_conf.available_services = available_services
         self.tempest_conf._configure_service_available()
 
         expected = (
             ("neutron", "False"), ("heat", "False"), ("nova", "True"),
             ("swift", "False"), ("cinder", "True"), ("sahara", "True"),
-            ("glance", "True"), ("ceilometer", "False"))
+            ("glance", "True"), ("ceilometer", "False"), ("aodh", "True"))
         result = self.tempest_conf.conf.items("service_available")
         for item in expected:
             self.assertIn(item, result)
@@ -370,10 +370,10 @@ class TempestResourcesContextTestCase(test.TestCase):
         self.assertEqual(mock_neutron_wrapper_create_network.call_count, 0)
 
     def test__create_tempest_roles(self):
-        role1 = CONF.role.swift_operator_role
-        role2 = CONF.role.swift_reseller_admin_role
-        role3 = CONF.role.heat_stack_owner_role
-        role4 = CONF.role.heat_stack_user_role
+        role1 = CONF.tempest.swift_operator_role
+        role2 = CONF.tempest.swift_reseller_admin_role
+        role3 = CONF.tempest.heat_stack_owner_role
+        role4 = CONF.tempest.heat_stack_user_role
 
         client = self.context.clients.verified_keystone()
         client.roles.list.return_value = [fakes.FakeRole(name=role1),
@@ -423,9 +423,9 @@ class TempestResourcesContextTestCase(test.TestCase):
         mock_wrap.assert_called_once_with(self.context.clients.glance,
                                           self.context)
         client.create_image.assert_called_once_with(
-            container_format=CONF.image.container_format,
+            container_format=CONF.tempest.img_container_format,
             image_location=mock.ANY,
-            disk_format=CONF.image.disk_format,
+            disk_format=CONF.tempest.img_disk_format,
             name=mock.ANY,
             visibility="public")
 
@@ -472,16 +472,24 @@ class TempestResourcesContextTestCase(test.TestCase):
         client = self.context.clients.keystone()
         self.assertEqual(client.roles.delete.call_count, 2)
 
-    def test__cleanup_images(self):
+    @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
+    def test__cleanup_images(self, mock_wrap):
         self.context._created_images = [fakes.FakeImage(id="id1"),
                                         fakes.FakeImage(id="id2")]
 
         self.context.conf.set("compute", "image_ref", "id1")
         self.context.conf.set("compute", "image_ref_alt", "id2")
 
+        wrapper = mock_wrap.return_value
+        wrapper.get_image.side_effect = [
+            fakes.FakeImage(id="id1", status="DELETED"),
+            fakes.FakeImage(id="id2"),
+            fakes.FakeImage(id="id2", status="DELETED")]
+
         self.context._cleanup_images()
         client = self.context.clients.glance()
-        self.assertEqual(client.images.delete.call_count, 2)
+        client.images.delete.assert_has_calls([mock.call("id1"),
+                                               mock.call("id2")])
 
         self.assertEqual("", self.context.conf.get("compute", "image_ref"))
         self.assertEqual("", self.context.conf.get("compute", "image_ref_alt"))
