@@ -17,6 +17,7 @@ import copy
 import os
 import subprocess
 
+import ddt
 import mock
 
 from rally import exceptions
@@ -94,22 +95,18 @@ class TempestUtilsTestCase(BaseTestCase):
         self.assertFalse(mock_check_output.called)
 
     @mock.patch("%s.tempest.sys" % TEMPEST_PATH)
-    @mock.patch("%s.tempest.costilius.get_interpreter" % TEMPEST_PATH,
-                return_value="python")
     @mock.patch("os.path.isdir", return_value=False)
     @mock.patch("%s.tempest.check_output" % TEMPEST_PATH,
                 return_value="some_output")
     def test__venv_install_when_venv_not_exist(self, mock_check_output,
-                                               mock_isdir,
-                                               mock_get_interpreter, mock_sys):
+                                               mock_isdir, mock_sys):
         mock_sys.version_info = "not_py27_env"
-
         self.verifier._install_venv()
 
         mock_isdir.assert_called_once_with(self.verifier.path(".venv"))
         mock_check_output.assert_has_calls([
-            mock.call(["virtualenv", "-p", mock_get_interpreter.return_value,
-                       ".venv"], cwd="/tmp"),
+            mock.call(["virtualenv", "-p", mock_sys.executable, ".venv"],
+                      cwd="/tmp"),
             mock.call(["/tmp/tools/with_venv.sh", "pip", "install", "-r",
                        "requirements.txt", "-r", "test-requirements.txt"],
                       cwd="/tmp"),
@@ -118,14 +115,13 @@ class TempestUtilsTestCase(BaseTestCase):
         ])
 
     @mock.patch("%s.tempest.sys" % TEMPEST_PATH)
-    @mock.patch("%s.tempest.costilius.get_interpreter" % TEMPEST_PATH,
-                return_value=None)
     @mock.patch("os.path.isdir", return_value=False)
     def test__venv_install_fails__when_py27_is_not_present(
-            self, mock_isdir, mock_get_interpreter, mock_sys):
+            self, mock_isdir, mock_sys):
         mock_sys.version_info = "not_py27_env"
+        mock_sys.executable = "fake_path"
 
-        self.assertRaises(exceptions.IncompatiblePythonVersion,
+        self.assertRaises(tempest.TempestSetupFailure,
                           self.verifier._install_venv)
 
         mock_isdir.assert_called_once_with(self.verifier.path(".venv"))
@@ -252,6 +248,7 @@ class TempestInstallAndUninstallTestCase(BaseTestCase):
                                 mock_tempest__initialize_testr,
                                 mock_tempest_base_repo):
         mock_tempest_base_repo.__get__ = mock.Mock(return_value="fake_dir")
+        self.verifier.version = "3f4c8d44"
         self.verifier.install()
 
         mock_tempest__is_git_repo.assert_called_once_with(
@@ -263,10 +260,7 @@ class TempestInstallAndUninstallTestCase(BaseTestCase):
             self.verifier.base_repo,
             self.verifier.path())
         cwd = self.verifier.path("tempest")
-        expected = [
-            mock.call(["git", "checkout", "master"], cwd=cwd),
-            mock.call(["git", "pull"], cwd=cwd),
-        ]
+        expected = [mock.call(["git", "checkout", "3f4c8d44"], cwd=cwd)]
         self.assertEqual(expected, mock_check_call.mock_calls)
         mock_tempest__install_venv.assert_called_once_with()
         mock_tempest__initialize_testr.assert_called_once_with()
@@ -291,6 +285,7 @@ class TempestInstallAndUninstallTestCase(BaseTestCase):
         mock_tempest_base_repo.__get__ = mock.Mock(return_value="fake_dir")
         mock_check_call.side_effect = subprocess.CalledProcessError(0, None)
 
+        self.verifier.version = "3f4c8d44"
         self.assertRaises(tempest.TempestSetupFailure, self.verifier.install)
 
         mock_tempest__is_git_repo.assert_called_once_with(
@@ -332,6 +327,22 @@ class TempestInstallAndUninstallTestCase(BaseTestCase):
             mock_move.assert_called_once_with(source, dest)
 
 
+@ddt.ddt
+class TempestInstallPluginsTestCase(BaseTestCase):
+
+    @mock.patch(TEMPEST_PATH + ".tempest.check_output")
+    @ddt.data("https://github.com/fake-plugin.git", "/tmp/fake-plugin")
+    def test_install_plugin(self, plugin_source, mock_tempest_check_output):
+        self.verifier.plugin_source = plugin_source
+        self.verifier.install_plugin()
+
+        cmd = [self.verifier.venv_wrapper, "pip", "install",
+               "--src", self.verifier.path("plugins"), "-e",
+               "git+{0}@master#egg={1}".format(plugin_source, "fake-plugin")]
+        mock_tempest_check_output.assert_called_with(cmd,
+                                                     cwd=self.verifier.path())
+
+
 class TempestVerifyTestCase(BaseTestCase):
     def _get_fake_call(self, testr_args,
                        concur_args="--parallel --concurrency 0"):
@@ -344,57 +355,24 @@ class TempestVerifyTestCase(BaseTestCase):
                 "testr_args": testr_args,
                 "log_file": self.verifier.path("subunit.stream")})
 
-    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.parse_results",
-                return_value=None)
-    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
-    @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
-    @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
     @mock.patch(TEMPEST_PATH + ".config.TempestConfig")
-    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
-                return_value=False)
-    def test_verify_not_configured(
-            self, mock_tempest_is_configured, mock_tempest_config,
-            mock_tempest_resources_context, mock_subprocess, mock_tempest_env,
-            mock_tempest_parse_results):
-
-        set_name = "compute"
-        fake_call = self._get_fake_call("tempest.api.%s" % set_name)
-
-        self.verifier.verify(set_name, None, None, None, 0, False)
-
-        self.assertEqual(2, mock_tempest_is_configured.call_count)
-        mock_tempest_config.assert_called_once_with(self.verifier.deployment)
-        mock_tempest_config.return_value.generate.assert_called_once_with(
-            self.verifier.config_file
-        )
-        self.verifier.verification.start_verifying.assert_called_once_with(
-            set_name)
-
-        mock_subprocess.check_call.assert_called_once_with(
-            fake_call, env=mock_tempest_env, cwd=self.verifier.path(),
-            shell=True)
-        mock_tempest_parse_results.assert_called_once_with(None, None)
+    def test_verify_no_tempest_config_exists(self, mock_tempest_config):
+        self.assertRaises(exceptions.NotFoundException, self.verifier.verify,
+                          "compute", None, None, None, 0, False)
 
     @mock.patch(TEMPEST_PATH + ".tempest.Tempest.parse_results",
                 return_value=None)
     @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
     @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
     @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
-    @mock.patch(TEMPEST_PATH + ".config.TempestConfig")
-    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
-                return_value=True)
-    def test_verify_when_tempest_configured(
-            self, mock_tempest_is_configured, mock_tempest_config,
-            mock_tempest_resources_context, mock_subprocess, mock_tempest_env,
-            mock_tempest_parse_results):
+    @mock.patch("os.path.isfile", return_value=True)
+    def test_verify_tempest_config_exists(
+            self, mock_isfile, mock_tempest_resources_context, mock_subprocess,
+            mock_tempest_env, mock_tempest_parse_results):
         set_name = "identity"
         fake_call = self._get_fake_call("tempest.api.%s" % set_name)
 
         self.verifier.verify(set_name, None, None, None, 0, False)
-
-        mock_tempest_is_configured.assert_called_once_with()
-        self.assertFalse(mock_tempest_config.called)
-        self.assertFalse(mock_tempest_config().generate.called)
         self.verifier.verification.start_verifying.assert_called_once_with(
             set_name)
 
@@ -408,29 +386,22 @@ class TempestVerifyTestCase(BaseTestCase):
     @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
     @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
     @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
-    @mock.patch(TEMPEST_PATH + ".config.TempestConfig")
-    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
-                return_value=True)
-    def test_verify_failed_and_tempest_is_configured(
-            self, mock_tempest_is_configured, mock_tempest_config,
-            mock_tempest_resources_context, mock_subprocess, mock_tempest_env,
-            mock_tempest_parse_results):
+    @mock.patch("os.path.isfile", return_value=True)
+    def test_verify_failed_and_tempest_config_exists(
+            self, mock_isfile, mock_tempest_resources_context, mock_subprocess,
+            mock_tempest_env, mock_tempest_parse_results):
         set_name = "identity"
         fake_call = self._get_fake_call("tempest.api.%s" % set_name)
         mock_subprocess.side_effect = subprocess.CalledProcessError
 
         self.verifier.verify(set_name, None, None, None, 0, False)
-
-        mock_tempest_is_configured.assert_called_once_with()
-        self.assertFalse(mock_tempest_config.called)
-        self.assertFalse(mock_tempest_config().generate.called)
         self.verifier.verification.start_verifying.assert_called_once_with(
             set_name)
 
         mock_subprocess.check_call.assert_called_once_with(
             fake_call, env=mock_tempest_env, cwd=self.verifier.path(),
             shell=True)
-        self.assertTrue(mock_tempest_parse_results.called)
+        mock_tempest_parse_results.assert_called_once_with(None, None)
         self.verifier.verification.set_failed.assert_called_once_with()
 
     @mock.patch(TEMPEST_PATH + ".tempest.Tempest.parse_results",
@@ -438,10 +409,9 @@ class TempestVerifyTestCase(BaseTestCase):
     @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
     @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
     @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
-    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
-                return_value=True)
+    @mock.patch("os.path.isfile", return_value=True)
     def test_verify_tests_file_specified(
-            self, mock_tempest_is_configured, mock_tempest_resources_context,
+            self, mock_isfile, mock_tempest_resources_context,
             mock_subprocess, mock_tempest_env, mock_tempest_parse_results):
         tests_file = "/path/to/tests/file"
         fake_call = self._get_fake_call("--load-list %s" % tests_file)
@@ -459,10 +429,9 @@ class TempestVerifyTestCase(BaseTestCase):
     @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
     @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
     @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
-    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
-                return_value=True)
+    @mock.patch("os.path.isfile", return_value=True)
     def test_verify_concurrency_equals_to_1(
-            self, mock_tempest_is_configured, mock_tempest_resources_context,
+            self, mock_isfile, mock_tempest_resources_context,
             mock_subprocess, mock_tempest_env, mock_tempest_parse_results):
         set_name = "identity"
         fake_call = self._get_fake_call(
@@ -482,10 +451,9 @@ class TempestVerifyTestCase(BaseTestCase):
     @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
     @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
     @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
-    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
-                return_value=True)
+    @mock.patch("os.path.isfile", return_value=True)
     def test_verify_concurrency_doesnt_equal_to_1(
-            self, mock_tempest_is_configured, mock_tempest_resources_context,
+            self, mock_isfile, mock_tempest_resources_context,
             mock_subprocess, mock_tempest_env, mock_tempest_parse_results):
         set_name = "identity"
         fake_call = self._get_fake_call("tempest.api.%s" % set_name)
@@ -504,9 +472,8 @@ class TempestVerifyTestCase(BaseTestCase):
     @mock.patch(TEMPEST_PATH + ".tempest.Tempest.env")
     @mock.patch(TEMPEST_PATH + ".tempest.subprocess")
     @mock.patch(TEMPEST_PATH + ".config.TempestResourcesContext")
-    @mock.patch(TEMPEST_PATH + ".tempest.Tempest.is_configured",
-                return_value=True)
-    def test_verify_run_failed_tests_(self, mock_tempest_is_configured,
+    @mock.patch("os.path.isfile", return_value=True)
+    def test_verify_run_failed_tests_(self, mock_isfile,
                                       mock_tempest_resources_context,
                                       mock_subprocess, mock_tempest_env,
                                       mock_tempest_parse_results):
